@@ -13,6 +13,7 @@ case "$osname-$cputype" in
   Darwin-*arm* )           platform=Silicon ;;
   CYGWIN_NT-* | MINGW*-* ) platform=CYGWIN_NT ;;
   Linux-*arm* )            platform=ARM ;;
+  Linux-aarch64 )          platform=ARM64 ;;
   * )                      platform=UNSUPPORTED ;;
 esac
 
@@ -94,7 +95,7 @@ then
 fi
 
 # erase any existing executables in current directory
-for plt in Darwin Silicon Linux CYGWIN_NT ARM
+for plt in Darwin Silicon Linux CYGWIN_NT ARM ARM64
 do
   rm -f *.$plt
 done
@@ -104,7 +105,8 @@ mods="darwin amd64 Darwin \
       darwin arm64 Silicon \
       linux amd64 Linux \
       windows 386 CYGWIN_NT \
-      linux arm ARM"
+      linux arm ARM \
+      linux arm64 ARM64"
 
 # build all executables for each selected platform
 echo "$mods" |
@@ -126,13 +128,13 @@ then
   if [ "$cleanup" = true ]
   then
     # remove old executables from target
-    for plt in Darwin Silicon Linux CYGWIN_NT ARM
+    for plt in Darwin Silicon Linux CYGWIN_NT ARM ARM64
     do
       rm -f $target/*.$plt
     done
   fi
   # copy new executables to target
-  for plt in Darwin Silicon Linux CYGWIN_NT ARM
+  for plt in Darwin Silicon Linux CYGWIN_NT ARM ARM64
   do
     for exc in xtract rchive transmute
     do
@@ -145,7 +147,7 @@ then
 fi
 
 # erase any remaining executables after compiling
-for plt in Darwin Silicon Linux CYGWIN_NT ARM
+for plt in Darwin Silicon Linux CYGWIN_NT ARM ARM64
 do
   rm -f *.$plt
 done
