@@ -32,7 +32,6 @@ package main
 
 import (
 	"eutils"
-	"fmt"
 	"github.com/gin-gonic/gin"
 	"net/http"
 	"os"
@@ -288,35 +287,25 @@ func main() {
 	// DATA AVAILABILITY REALITY CHECKS
 
 	// obtain path from environment variable
-	base := os.Getenv("EDIRECT_PUBMED_MASTER")
-	if base != "" {
-		if !strings.HasSuffix(base, "/") {
-			base += "/"
-		}
-	}
-	if base == "" {
-		fmt.Fprintf(os.Stderr, "\nERROR: Local archive path is not specified\n\n")
-		os.Exit(1)
-	}
 
-	archiveBase := base + "Archive"
-	postingsBase := base + "Postings"
-	dataBase := base + "Data"
+	archiveBase := eutils.GetLocalArchiveFolder("pubmed", "Archive", false)
+	postingsBase := eutils.GetLocalArchiveFolder("pubmed", "Postings", false)
+	dataBase := eutils.GetLocalArchiveFolder("pubmed", "Data", false)
 
 	// check to make sure local archive and search index directories are mounted
 	_, err := os.Stat(archiveBase)
 	if err != nil && os.IsNotExist(err) {
-		fmt.Fprintf(os.Stderr, "\nERROR: Local archive is not mounted\n\n")
+		eutils.DisplayError("Local archive is not mounted")
 		os.Exit(1)
 	}
 	_, err = os.Stat(postingsBase)
 	if err != nil && os.IsNotExist(err) {
-		fmt.Fprintf(os.Stderr, "\nERROR: Local search index is not mounted\n\n")
+		eutils.DisplayError("Local search index is not mounted")
 		os.Exit(1)
 	}
 	_, err = os.Stat(dataBase)
 	if err != nil && os.IsNotExist(err) {
-		fmt.Fprintf(os.Stderr, "\nERROR: Local mapping data is not mounted\n\n")
+		eutils.DisplayError("Local mapping data is not mounted")
 		os.Exit(1)
 	}
 
@@ -398,11 +387,11 @@ func main() {
 
 		// concurrent fetching by multiple goroutines
 		uidq := eutils.ReadsUIDsFromString(uids)
-		strq := eutils.CreateFetchers(archiveBase, "pubmed", "", ".xml", true, uidq)
+		strq := eutils.CreateFetchers(archiveBase, "pubmed", "", ".xml", "PubmedArticle", true, uidq)
 		unsq := eutils.CreateXMLUnshuffler(strq)
 
 		if uidq == nil || strq == nil || unsq == nil {
-			fmt.Fprintf(os.Stderr, "\nERROR: Unable to create archive reader\n")
+			eutils.DisplayError("Unable to create archive reader")
 			os.Exit(1)
 		}
 
@@ -478,11 +467,11 @@ func main() {
 
 		// concurrent fetching by multiple goroutines
 		uidq := eutils.ReadsUIDsFromString(uids)
-		strq := eutils.CreateCacheStreamers(archiveBase, "", ".xml", uidq)
+		strq := eutils.CreateCacheStreamers(archiveBase, "pubmed", "", ".xml", 0, uidq)
 		unsq := eutils.CreateXMLUnshuffler(strq)
 
 		if uidq == nil || strq == nil || unsq == nil {
-			fmt.Fprintf(os.Stderr, "\nERROR: Unable to create archive steamer\n")
+			eutils.DisplayError("Unable to create archive steamer")
 			os.Exit(1)
 		}
 
@@ -526,7 +515,7 @@ func main() {
 	// common search function
 	pubmedSearch := func(c *gin.Context, query string) {
 
-		uids := eutils.ProcessQuery(postingsBase, "pubmed", query, false, false, false, false, deStop)
+		uids := eutils.ProcessQuery("pubmed", query, false, false, false, deStop)
 
 		// use buffer to speed up uid printing
 		var buffer strings.Builder
@@ -567,7 +556,7 @@ func main() {
 	cache := eutils.NewCitCache(500)
 
 	if cache == nil {
-		fmt.Fprintf(os.Stderr, "\nERROR: Unable to create citation matcher cache\n")
+		eutils.DisplayError("Unable to create citation matcher cache")
 		os.Exit(1)
 	}
 
@@ -659,13 +648,13 @@ func main() {
 		}
 
 		sgr := strings.NewReader(cit)
-		rdr := eutils.CreateXMLStreamer(sgr)
+		rdr := eutils.CreateXMLStreamer(sgr, nil)
 		xmlq := eutils.CreateXMLProducer("CITATION", "", false, rdr)
 		ctmq := eutils.CreateCitMatchers(xmlq, []string{"strict,remote,verify"}, deStop, doStem, cache, jtaMap)
 		unsq := eutils.CreateXMLUnshuffler(ctmq)
 
 		if sgr == nil || rdr == nil || xmlq == nil || ctmq == nil || unsq == nil {
-			fmt.Fprintf(os.Stderr, "\nERROR: Unable to create citation matcher\n")
+			eutils.DisplayError("Unable to create citation matcher")
 			os.Exit(1)
 		}
 
@@ -716,7 +705,7 @@ func main() {
 	// journal lookup from jtaMap
 	lookupJournal := func(c *gin.Context, query string) {
 
-		query = eutils.CleanJournal(query)
+		query = eutils.NormalizeJournal(query)
 		if query != "" {
 			query = strings.ToLower(query)
 			jta, ok := jtaMap[query]
