@@ -2620,7 +2620,7 @@ func main() {
 				if eutils.HasUnicodeMarkup(ctx) {
 					ctx = eutils.RepairUnicodeMarkup(ctx, eutils.SPACE)
 				}
-				ctx = eutils.RepairEncodedMarkup(ctx)
+				ctx = eutils.CleanupEncodedMarkup(ctx)
 				buffer.WriteString("\t| ")
 				buffer.WriteString(ctx)
 				if eutils.HasAmpOrNotASCII(ctx) {
