@@ -103,12 +103,6 @@ func main() {
 		os.Exit(1)
 	}
 
-	// pretty-printing of xtract arguments
-	if args[0] == "-pretty" {
-		eutils.PrettyArguments(args[1:])
-		return
-	}
-
 	// performance arguments
 	chanDepth := 0
 	farmSize := 0
@@ -351,6 +345,14 @@ func main() {
 
 	eutils.SetOptions(doStrict, doMixed, doSelf, deAccent, deSymbol, doASCII, doCompress, doCleanup, doStem, deStop)
 
+	// pretty-printing of xtract arguments
+	if args[0] == "-pretty" {
+
+		eutils.PrettyArguments(args[1:])
+
+		return
+	}
+
 	// -stats prints number of CPUs and performance tuning values if no other arguments (undocumented)
 	if stts && len(args) < 1 {
 
@@ -585,6 +587,9 @@ func main() {
 	hd := ""
 	tl := ""
 
+	xmltype := ""
+	doctype := ""
+
 	for {
 
 		inSwitch = true
@@ -671,6 +676,18 @@ func main() {
 				hd = "<" + tmp + ">"
 				tl = "</" + tmp + ">"
 			}
+		case "-xml":
+			if len(args) < 2 {
+				eutils.DisplayError("Pattern missing after -xml command")
+				os.Exit(1)
+			}
+			xmltype = eutils.CleanXmltype(args[1])
+		case "-doctype":
+			if len(args) < 2 {
+				eutils.DisplayError("Pattern missing after -doctype command")
+				os.Exit(1)
+			}
+			doctype = eutils.CleanDoctype(args[1])
 		default:
 			// if not any of the controls, set flag to break out of for loop
 			inSwitch = false
@@ -689,6 +706,17 @@ func main() {
 		}
 	}
 
+	if head != "" {
+		// first prepend doctype
+		if doctype != "" {
+			head = doctype + "\n" + head
+		}
+		// then prepend xml line
+		if xmltype != "" {
+			head = xmltype + "\n" + head
+		}
+	}
+
 	// CREATE XML BLOCK READER FROM STDIN OR FILE
 
 	const FirstBuffSize = 4096
@@ -1455,23 +1483,27 @@ func main() {
 		return
 	}
 
-	// SPLIT FILE BY BY RECORD COUNT
+	// SPLIT FILE BY RECORD COUNT
 
 	// split XML record into subfiles by count
 	if len(args) == 8 && args[2] == "-split" && args[4] == "-prefix" && args[6] == "-suffix" {
 
 		// e.g., -head "<IdxDocumentSet>" -tail "</IdxDocumentSet>" -pattern IdxDocument -split 250000 -prefix "biocon" -suffix "e2x"
+
 		count := 0
 		fnum := 0
+
 		var (
 			fl  *os.File
 			err error
 		)
+
 		chunk, err := strconv.Atoi(args[3])
 		if err != nil {
 			eutils.DisplayError("-split argument '%s' is not an integer", err.Error())
 			return
 		}
+
 		prefix := args[5]
 		suffix := args[7]
 
@@ -1524,6 +1556,90 @@ func main() {
 		return
 	}
 
+	// ALLOT RECORDS BY BYTE SIZE
+
+	// allot XML record into subfiles by size
+	if len(args) == 8 && args[2] == "-allot" && args[4] == "-prefix" && args[6] == "-suffix" {
+
+		// e.g., -head "<IdxDocumentSet>" -tail "</IdxDocumentSet>" -pattern IdxDocument -allot 10000 -prefix "biocon" -suffix "e2x"
+
+		retlen := len("\n")
+		cumulative := 0
+		fnum := 0
+
+		headlen := 0
+		taillen := 0
+		if head != "" {
+			headlen += len(head) + retlen
+		}
+		if tail != "" {
+			taillen += len(tail) + retlen
+		}
+		padlen := retlen + headlen + taillen
+
+		var (
+			fl  *os.File
+			err error
+		)
+
+		chunk, err := strconv.Atoi(args[3])
+		if err != nil {
+			eutils.DisplayError("-allot argument '%s' is not an integer", err.Error())
+			return
+		}
+
+		prefix := args[5]
+		suffix := args[7]
+
+		eutils.PartitionXML(topPattern, star, false, rdr,
+			func(str string) {
+				recordCount++
+
+				if cumulative >= chunk {
+					if tail != "" {
+						fl.WriteString(tail)
+						fl.WriteString("\n")
+					}
+					fl.Close()
+					cumulative = 0
+				}
+				if cumulative == 0 {
+					fpath := fmt.Sprintf("%s%03d.%s", prefix, fnum, suffix)
+					fl, err = os.Create(fpath)
+					if err != nil {
+						eutils.DisplayError("Unable to create path '%s'", err.Error())
+						return
+					}
+					os.Stderr.WriteString(fpath + "\n")
+					fnum++
+					if head != "" {
+						fl.WriteString(head)
+						fl.WriteString("\n")
+					}
+				}
+				cumulative += len(str) + padlen
+
+				fl.WriteString(str[:])
+				fl.WriteString("\n")
+			})
+
+		if cumulative > 0 {
+			if tail != "" {
+				fl.WriteString(tail)
+				fl.WriteString("\n")
+			}
+			fl.Close()
+		}
+
+		debug.FreeOSMemory()
+
+		if timr {
+			printDuration("records")
+		}
+
+		return
+	}
+
 	// PARSE AND VALIDATE EXTRACTION ARGUMENTS
 
 	// parse nested exploration instruction from command-line arguments
