@@ -92,6 +92,9 @@ func AlignColumns(inp io.Reader, margin, padding, minimum int, align string) <-c
 
 	if padding > 0 && padding < 30 {
 		pad = spaces[0:padding]
+	} else if padding == -1 {
+		// continue using tab character between columns
+		pad = "\t"
 	}
 
 	align = strings.TrimSpace(align)
