@@ -32,12 +32,12 @@ package eutils
 
 import (
 	"bufio"
-	"fmt"
 	"html"
 	"io"
 	"os"
 	"runtime"
 	"strings"
+	"unicode"
 )
 
 // ASN1Converter parses text ASN.1 records into XML objects
@@ -50,7 +50,7 @@ func ASN1Converter(inp io.Reader, set, rec string) <-chan string {
 	tks := make(chan string, chanDepth)
 	out := make(chan string, chanDepth)
 	if tks == nil || out == nil {
-		fmt.Fprintf(os.Stderr, "\nERROR: Unable to create ASN1 converter channels\n")
+		DisplayError("Unable to create ASN1 converter channels")
 		os.Exit(1)
 	}
 
@@ -80,6 +80,15 @@ func ASN1Converter(inp io.Reader, set, rec string) <-chan string {
 					// ignore blank lines
 					continue
 				}
+
+				// https://stackoverflow.com/questions/58994146/how-to-remove-non-printable-characters
+				line = strings.Map(func(r rune) rune {
+					if unicode.IsPrint(r) {
+						return r
+					}
+					return -1
+				}, line)
+
 				// add sentinel
 				line += sentinel
 				return line
@@ -91,7 +100,6 @@ func ASN1Converter(inp io.Reader, set, rec string) <-chan string {
 
 		readRestOfAsnString := func() string {
 
-			// continue reading additional lines of string
 			for {
 
 				line = nextLine()
@@ -130,7 +138,6 @@ func ASN1Converter(inp io.Reader, set, rec string) <-chan string {
 
 		readRestOfAsnBits := func() string {
 
-			// continue reading additional lines of string
 			for {
 
 				line = nextLine()
@@ -160,7 +167,7 @@ func ASN1Converter(inp io.Reader, set, rec string) <-chan string {
 					return line
 				}
 
-				// string continues on additional lines
+				// bit string continues on additional lines
 			}
 
 			return line
@@ -554,7 +561,7 @@ func ASN1Converter(inp io.Reader, set, rec string) <-chan string {
 					}
 					return
 				case "::=":
-					fmt.Fprintf(os.Stderr, "\nERROR: Unexpected ::= token found\n")
+					DisplayError("Unexpected ::= token found")
 					os.Exit(1)
 				default:
 					arry = append(arry, tkn)
@@ -594,11 +601,11 @@ func ASN1Converter(inp io.Reader, set, rec string) <-chan string {
 
 			tkn := nextToken()
 			if tkn == "" {
-				fmt.Fprintf(os.Stderr, "\nERROR: Incomplete ASN1 starting with '%s'\n", top)
+				DisplayError("Incomplete ASN1 starting with '%s'", top)
 				os.Exit(1)
 			}
 			if tkn != "::=" {
-				fmt.Fprintf(os.Stderr, "\nERROR: ASN1 message missing expected ::= token, found '%s'\n", tkn)
+				DisplayError("ASN1 message missing expected ::= token, found '%s'", tkn)
 				os.Exit(1)
 			}
 
@@ -629,35 +636,10 @@ func ASN1Converter(inp io.Reader, set, rec string) <-chan string {
 	return out
 }
 
-// ASNtoXML sends converted XML to a callback
+// ASNtoXML converts ASN.1 to an XML string
 func ASNtoXML(asn, set, rec string) string {
 
-	if asn == "" {
-		return ""
-	}
-
-	acnv := ASN1Converter(strings.NewReader(asn), set, rec)
-	if acnv == nil {
-		fmt.Fprintf(os.Stderr, "Unable to create ASN.1 converter\n")
-		return ""
-	}
-
-	var arry []string
-
-	// drain output of channel
-	for str := range acnv {
-		if str == "" {
-			continue
-		}
-
-		if !strings.HasSuffix(str, "\n") {
-			str += "\n"
-		}
-
-		arry = append(arry, str)
-	}
-
-	res := strings.Join(arry, "")
-
-	return res
+	return StringToXML(asn, func(inp io.Reader) <-chan string {
+		return ASN1Converter(inp, set, rec)
+	})
 }
