@@ -117,79 +117,39 @@ func fetchOneXMLRecord(id, base, pfx, sfx string, zipp bool, buf bytes.Buffer) s
 	return str
 }
 
-// FetchPubMedRecord returns the PubmedArticle XML for a single PMID
-func FetchPubMedRecord(id string) string {
+// FetchLocalRecord returns the XML for a single UID
+func FetchLocalRecord(id, db, pfx, ptrn string) string {
 
-	if id == "" {
+	if id == "" || db == "" {
 		return ""
 	}
 
+	if pfx != "" {
+		id = strings.TrimPrefix(id, pfx)
+	}
+
 	var buf bytes.Buffer
 
 	// obtain path from environment variable
-	base := os.Getenv("EDIRECT_PUBMED_MASTER")
-	if base != "" {
-		if !strings.HasSuffix(base, "/") {
-			base += "/"
-		}
-	}
+	archiveBase := GetLocalArchiveFolder(db, "Archive", true)
 
-	archiveBase := base + "Archive"
+	if archiveBase == "" {
+		DisplayError("Unable to get local archive path")
+		os.Exit(1)
+	}
 
 	// check to make sure local archive is mounted
 	_, err := os.Stat(archiveBase)
 	if err != nil && os.IsNotExist(err) {
-		fmt.Fprintf(os.Stderr, "\nERROR: Local archive and search index is not mounted\n\n")
+		DisplayError("Local archive and search index is not mounted")
 		os.Exit(1)
 	}
 
-	str := fetchOneXMLRecord(id, archiveBase, "", ".xml", true, buf)
+	str := fetchOneXMLRecord(id, archiveBase, pfx, ".xml", true, buf)
 
-	// trim header now included in archive XML files
-	if str != "" {
-		pos := strings.Index(str, "<PubmedArticle>")
-		if pos > 0 {
-			// remove leading xml and DOCTYPE lines
-			str = str[pos:]
-		}
-	}
-
-	return str
-}
-
-// FetchPMCRecord returns the PMCExtract XML for a single PMCID
-func FetchPMCRecord(id string) string {
-
-	if id == "" {
-		return ""
-	}
-
-	id = strings.TrimPrefix(id, "PMC")
-
-	var buf bytes.Buffer
-
-	// obtain path from environment variable
-	base := os.Getenv("EDIRECT_PMC_MASTER")
-	if base != "" {
-		if !strings.HasSuffix(base, "/") {
-			base += "/"
-		}
-	}
-
-	archiveBase := base + "Archive"
-
-	// check to make sure local archive is mounted
-	_, err := os.Stat(archiveBase)
-	if err != nil && os.IsNotExist(err) {
-		fmt.Fprintf(os.Stderr, "\nERROR: Local archive and search index is not mounted\n\n")
-		os.Exit(1)
-	}
-
-	str := fetchOneXMLRecord(id, archiveBase, "PMC", ".xml", true, buf)
-
-	// trim header now included in archive XML files
-	if str != "" {
-		pos := strings.Index(str, "<PMCExtract>")
+	// trim any header included in archive XML file
+	if str != "" && ptrn != "" {
+		pos := strings.Index(str, "<"+ptrn+">")
 		if pos > 0 {
 			// remove any leading xml and DOCTYPE lines
 			str = str[pos:]
@@ -230,9 +190,9 @@ func CreateUIDReader(in io.Reader) <-chan XMLRecord {
 		return nil
 	}
 
-	out := make(chan XMLRecord, ChanDepth())
+	out := make(chan XMLRecord, chanDepth)
 	if out == nil {
-		fmt.Fprintf(os.Stderr, "\nERROR: Unable to create uid reader channel\n")
+		DisplayError("Unable to create uid reader channel")
 		os.Exit(1)
 	}
 
@@ -274,9 +234,9 @@ func ReadsUIDsFromString(uids string) <-chan XMLRecord {
 		return nil
 	}
 
-	out := make(chan XMLRecord, ChanDepth())
+	out := make(chan XMLRecord, chanDepth)
 	if out == nil {
-		fmt.Fprintf(os.Stderr, "\nERROR: Unable to create uid string reader channel\n")
+		DisplayError("Unable to create uid string reader channel")
 		os.Exit(1)
 	}
 
@@ -368,9 +328,9 @@ func mapXMLtoASN(node *XMLNode, proc func(string)) {
 			name = "_"
 		}
 
-		// leading hyphen  - unnamed braces (element of SEQUENCE OF or SET OF structured objects)
-		// trailing hyphen - unquoted value
-		// internal hyphen - convert to space
+		// leading underscore  - unnamed braces (element of SEQUENCE OF or SET OF structured objects)
+		// trailing underscore - unquoted value
+		// internal underscore - convert to space
 		show := true
 		quot := true
 		if strings.HasPrefix(name, "_") {
@@ -442,15 +402,15 @@ const XMLDoctypeGzipLen = 183
 
 // CreateStashers saves records to archive, multithreaded for performance, use of UID
 // position index allows it to prevent earlier version from overwriting later version
-func CreateStashers(stsh, parent, indx, pfx, sfx string, hash, zipp, asn, pmc bool, report int, inp <-chan XMLRecord) <-chan string {
+func CreateStashers(stsh, parent, indx, pfx, sfx, db, xmlString string, transform map[string]string, hash, zipp, asn bool, report int, inp <-chan XMLRecord) <-chan string {
 
 	if inp == nil {
 		return nil
 	}
 
-	out := make(chan string, ChanDepth())
+	out := make(chan string, chanDepth)
 	if out == nil {
-		fmt.Fprintf(os.Stderr, "\nERROR: Unable to create stasher channel\n")
+		DisplayError("Unable to create stasher channel")
 		os.Exit(1)
 	}
 
@@ -529,26 +489,14 @@ func CreateStashers(stsh, parent, indx, pfx, sfx string, hash, zipp, asn, pmc bo
 		tlock.Unlock()
 	}
 
-	// common xml + DOCTYPE header for PubmedArticle XML
-	pmaSetHead := `<?xml version="1.0" encoding="UTF-8"?>
-<!DOCTYPE PubmedArticle PUBLIC "-//NLM//DTD PubMedArticle, 1st January 2019//EN" "https://dtd.nlm.nih.gov/ncbi/pubmed/out/pubmed_190101.dtd">
-`
-
-	// common xml without DOCTYPE header for PMCExtract XML (derived from BioC)
-	pmcSetHead := `<?xml version="1.0" encoding="UTF-8"?>
-`
-
-	xmlString := ""
-	if pmc {
-		xmlString = pmcSetHead
-	} else {
-		xmlString = pmaSetHead
-	}
+	var xmlDoctype []byte
 
 	// make gzip-compressed byte array of xml + DOCTYPE header
-	xmlDoctype := GzipString(xmlString)
+	if xmlString != "" {
+		xmlDoctype = GzipString(xmlString)
+	}
 
-	if !pmc {
+	if db == "pubmed" {
 
 		// reality check on expected length
 		pmaHeadLen := len(xmlDoctype)
@@ -595,7 +543,7 @@ func CreateStashers(stsh, parent, indx, pfx, sfx string, hash, zipp, asn, pmc bo
 				}
 			}
 			str = txt
-		} else if pmc {
+		} else if db == "pmc" {
 			// for XML, add trailing newline, needed for streaming without decompressing
 			if !strings.HasSuffix(str, "\n") {
 				str += "\n"
@@ -622,7 +570,7 @@ func CreateStashers(stsh, parent, indx, pfx, sfx string, hash, zipp, asn, pmc bo
 				attempts--
 				if attempts < 1 {
 					// could not get lock after several attempts
-					fmt.Fprintf(os.Stderr, "\nERROR: Unable to save '%s'\n", id)
+					DisplayError("Unable to save '%s'", id)
 					return ""
 				}
 			case BAIL:
@@ -668,7 +616,7 @@ func CreateStashers(stsh, parent, indx, pfx, sfx string, hash, zipp, asn, pmc bo
 
 		if zipp {
 
-			if !asn {
+			if !asn && db == "pubmed" {
 				fl.Write(xmlDoctype)
 			}
 
@@ -705,7 +653,15 @@ func CreateStashers(stsh, parent, indx, pfx, sfx string, hash, zipp, asn, pmc bo
 
 		for ext := range inp {
 
-			ext.Ident = FindIdentifier(ext.Text, parent, find)
+			ident := FindIdentifier(ext.Text, parent, find)
+
+			// optional lookup of integer identifier from accession.version string
+			txt, found := transform[ident]
+			if found {
+				ident = txt
+			}
+
+			ext.Ident = ident
 
 			// skip BioC records with explicit 'unknown' identifier
 			if ext.Ident == "unknown" {
@@ -729,7 +685,7 @@ func CreateStashers(stsh, parent, indx, pfx, sfx string, hash, zipp, asn, pmc bo
 	var wg sync.WaitGroup
 
 	// launch multiple stasher goroutines
-	for i := 0; i < NumServe(); i++ {
+	for range numServe {
 		wg.Add(1)
 		go xmlStasher(&wg, inp, out)
 	}
@@ -747,15 +703,15 @@ func CreateStashers(stsh, parent, indx, pfx, sfx string, hash, zipp, asn, pmc bo
 
 // CreateDeleter reads PMIDs, deletes them in the archive, and sends them
 // down a channel to have the affected inverted index cache files removed.
-func CreateDeleter(stsh string, in io.Reader) <-chan string {
+func CreateDeleter(stsh, db string, in io.Reader) <-chan string {
 
 	if stsh == "" || in == nil {
 		return nil
 	}
 
-	out := make(chan string, ChanDepth())
+	out := make(chan string, chanDepth)
 	if out == nil {
-		fmt.Fprintf(os.Stderr, "\nERROR: Unable to create deleter channel\n")
+		DisplayError("Unable to create deleter channel")
 		os.Exit(1)
 	}
 
@@ -768,8 +724,8 @@ func CreateDeleter(stsh string, in io.Reader) <-chan string {
 
 		verbose := false
 		// set verbose flag from environment variable
-		env := os.Getenv("EDIRECT_PUBMED_VERBOSE")
-		if env == "Y" || env == "y" {
+		env := os.Getenv("EDIRECT_LOCAL_VERBOSE")
+		if env == "Y" || env == "y" || env == "true" {
 			verbose = true
 		}
 
@@ -814,15 +770,15 @@ func CreateDeleter(stsh string, in io.Reader) <-chan string {
 }
 
 // CreateClearer clears incremental index files whose underlying records have changed
-func CreateClearer(indexBase, invertBase string, inp <-chan string) <-chan string {
+func CreateClearer(indexBase, invertBase, db string, inp <-chan string) <-chan string {
 
 	if inp == nil {
 		return nil
 	}
 
-	out := make(chan string, ChanDepth())
+	out := make(chan string, chanDepth)
 	if out == nil {
-		fmt.Fprintf(os.Stderr, "\nERROR: Unable to create clearer channel\n")
+		DisplayError("Unable to create clearer channel")
 		os.Exit(1)
 	}
 
@@ -838,8 +794,8 @@ func CreateClearer(indexBase, invertBase string, inp <-chan string) <-chan strin
 
 		verbose := false
 		// set verbose flag from environment variable
-		env := os.Getenv("EDIRECT_PUBMED_VERBOSE")
-		if env == "Y" || env == "y" {
+		env := os.Getenv("EDIRECT_LOCAL_VERBOSE")
+		if env == "Y" || env == "y" || env == "true" {
 			verbose = true
 		}
 
@@ -854,21 +810,21 @@ func CreateClearer(indexBase, invertBase string, inp <-chan string) <-chan strin
 				id = id[:pos]
 			}
 
-			dir, idx := IndexTrie(id)
-			if dir == "" || idx == "" {
+			xdir, idx := IndexTrie(id)
+			if xdir == "" || idx == "" {
 				continue
 			}
 
-			dpath := filepath.Join(indexBase, dir, idx+".e2x.gz")
-			deletedIdx[dpath] = true
+			xpath := filepath.Join(indexBase, xdir, idx+".e2x.gz")
+			deletedIdx[xpath] = true
 
-			inv := InvertTrie(id)
-			if inv == "" {
+			vdir, inv := InvertTrie(id)
+			if vdir == "" || inv == "" {
 				continue
 			}
 
-			dpath = filepath.Join(invertBase, inv+".inv.gz")
-			deletedInv[dpath] = true
+			vpath := filepath.Join(invertBase, vdir, inv+".inv.gz")
+			deletedInv[vpath] = true
 
 			out <- id
 		}
@@ -898,15 +854,30 @@ func CreateClearer(indexBase, invertBase string, inp <-chan string) <-chan strin
 }
 
 // CreateFetchers returns uncompressed records from archive, multithreaded for speed
-func CreateFetchers(stsh, db, pfx, sfx string, zipp bool, inp <-chan XMLRecord) <-chan XMLRecord {
+func CreateFetchers(stsh, db, pfx, sfx, ptrn string, zipp bool, inp <-chan XMLRecord) <-chan XMLRecord {
 
 	if inp == nil || stsh == "" {
 		return nil
 	}
 
-	out := make(chan XMLRecord, ChanDepth())
+	// obtain path from environment variable
+	archiveBase := GetLocalArchiveFolder(db, "Archive", true)
+
+	if archiveBase == "" {
+		DisplayError("Unable to get local archive path")
+		os.Exit(1)
+	}
+
+	// check to make sure local archive is mounted
+	_, err := os.Stat(archiveBase)
+	if err != nil && os.IsNotExist(err) {
+		DisplayError("Local archive is not mounted")
+		os.Exit(1)
+	}
+
+	out := make(chan XMLRecord, chanDepth)
 	if out == nil {
-		fmt.Fprintf(os.Stderr, "\nERROR: Unable to create fetcher channel\n")
+		DisplayError("Unable to create fetcher channel")
 		os.Exit(1)
 	}
 
@@ -924,22 +895,12 @@ func CreateFetchers(stsh, db, pfx, sfx string, zipp bool, inp <-chan XMLRecord)
 
 			str := fetchOneXMLRecord(ext.Text, stsh, pfx, sfx, zipp, buf)
 
-			// trim header now included in archive XML files
-			if db == "" || db == "pubmed" {
-				if str != "" {
-					pos := strings.Index(str, "<PubmedArticle>")
-					if pos > 0 {
-						// remove leading xml and DOCTYPE lines
-						str = str[pos:]
-					}
-				}
-			} else if db == "pmc" {
-				if str != "" {
-					pos := strings.Index(str, "<PMCExtract>")
-					if pos > 0 {
-						// remove any leading xml and DOCTYPE lines
-						str = str[pos:]
-					}
+			// trim any header included in archive XML file
+			if str != "" && ptrn != "" {
+				pos := strings.Index(str, "<"+ptrn+">")
+				if pos > 0 {
+					// remove any leading xml and DOCTYPE lines
+					str = str[pos:]
 				}
 			}
 
@@ -952,7 +913,7 @@ func CreateFetchers(stsh, db, pfx, sfx string, zipp bool, inp <-chan XMLRecord)
 	var wg sync.WaitGroup
 
 	// launch multiple fetcher goroutines
-	for i := 0; i < NumServe(); i++ {
+	for range numServe {
 		wg.Add(1)
 		go xmlFetcher(&wg, inp, out)
 	}
@@ -968,15 +929,30 @@ func CreateFetchers(stsh, db, pfx, sfx string, zipp bool, inp <-chan XMLRecord)
 
 // CreateCacheStreamers returns compressed records from archive, multithreaded for speed,
 // could be used for sending records over network to be decompressed later by client
-func CreateCacheStreamers(stsh, pfx, sfx string, inp <-chan XMLRecord) <-chan XMLRecord {
+func CreateCacheStreamers(stsh, db, pfx, sfx string, skip int, inp <-chan XMLRecord) <-chan XMLRecord {
 
 	if inp == nil || stsh == "" {
 		return nil
 	}
 
-	out := make(chan XMLRecord, ChanDepth())
+	// obtain path from environment variable
+	archiveBase := GetLocalArchiveFolder(db, "Archive", true)
+
+	if archiveBase == "" {
+		DisplayError("Unable to get local archive path")
+		os.Exit(1)
+	}
+
+	// check to make sure local archive is mounted
+	_, err := os.Stat(archiveBase)
+	if err != nil && os.IsNotExist(err) {
+		DisplayError("Local archive is not mounted")
+		os.Exit(1)
+	}
+
+	out := make(chan XMLRecord, chanDepth)
 	if out == nil {
-		fmt.Fprintf(os.Stderr, "\nERROR: Unable to create streamer channel\n")
+		DisplayError("Unable to create streamer channel")
 		os.Exit(1)
 	}
 	sfx += ".gz"
@@ -1037,8 +1013,14 @@ func CreateCacheStreamers(stsh, pfx, sfx string, inp <-chan XMLRecord) <-chan XM
 			runtime.Gosched()
 
 			// skip past first gzip packet with xml and DOCTYPE
-			if len(data) > XMLDoctypeGzipLen {
-				data = data[XMLDoctypeGzipLen:]
+			if skip > 0 {
+				if len(data) > skip {
+					data = data[skip:]
+				}
+			} else if db == "pubmed" {
+				if len(data) > XMLDoctypeGzipLen {
+					data = data[XMLDoctypeGzipLen:]
+				}
 			}
 
 			out <- XMLRecord{Index: ext.Index, Ident: ext.Ident, Data: data}
@@ -1048,7 +1030,7 @@ func CreateCacheStreamers(stsh, pfx, sfx string, inp <-chan XMLRecord) <-chan XM
 	var wg sync.WaitGroup
 
 	// launch multiple streamer goroutines
-	for i := 0; i < NumServe(); i++ {
+	for range numServe {
 		wg.Add(1)
 		go xmlStreamer(&wg, inp, out)
 	}
