@@ -41,7 +41,7 @@ import (
 	"net/url"
 	"os"
 	"regexp"
-	"sort"
+	"slices"
 	"strconv"
 	"strings"
 	"sync"
@@ -256,7 +256,7 @@ func genBankRefIndex(inp io.Reader, deStop, doStem bool) <-chan string {
 
 	out := make(chan string, chanDepth)
 	if out == nil {
-		fmt.Fprintf(os.Stderr, "Unable to create GenBank converter channel\n")
+		DisplayError("Unable to create GenBank converter channel")
 		os.Exit(1)
 	}
 
@@ -947,7 +947,7 @@ func emblRefIndex(inp io.Reader, deStop, doStem bool) <-chan string {
 
 	out := make(chan string, chanDepth)
 	if out == nil {
-		fmt.Fprintf(os.Stderr, "Unable to create EMBL converter channel\n")
+		DisplayError("Unable to create EMBL converter channel")
 		os.Exit(1)
 	}
 
@@ -1340,7 +1340,7 @@ func GenBankRefIndex(inp io.Reader, deStop, doStem bool) <-chan string {
 		buffer := make([]byte, FirstBuffSize)
 		n, err := inp.Read(buffer)
 		if err != nil && err != io.EOF {
-			fmt.Fprintf(os.Stderr, "\nERROR: Unable to read first block: %s\n", err.Error())
+			DisplayError("Unable to read first block: %s", err.Error())
 			// os.Exit(1)
 		}
 		bufr := buffer[:n]
@@ -1485,9 +1485,9 @@ func CreateCitMatchers(inp <-chan XMLRecord, options []string, deStop, doStem bo
 		return nil
 	}
 
-	out := make(chan XMLRecord, ChanDepth())
+	out := make(chan XMLRecord, chanDepth)
 	if out == nil {
-		fmt.Fprintf(os.Stderr, "\nERROR: Unable to create citmatch channel\n")
+		DisplayError("Unable to create citmatch channel")
 		os.Exit(1)
 	}
 
@@ -1529,34 +1529,43 @@ func CreateCitMatchers(inp <-chan XMLRecord, options []string, deStop, doStem bo
 			case "slow", "slower":
 				slower = true
 			default:
-				fmt.Fprintf(os.Stderr, "\nERROR: Unrecognized -options choice '%s'\n", opt)
+				DisplayError("Unrecognized -options choice '%s'", opt)
 				os.Exit(1)
 			}
 		}
 	}
 
-	postingsBase := ""
+	// archiveBase needed by eutisl.fetchOneXMLRecord function
 	archiveBase := ""
 
 	if local {
 
-		// obtain path from environment variable
-		base := os.Getenv("EDIRECT_PUBMED_MASTER")
-		if base != "" {
-			if !strings.HasSuffix(base, "/") {
-				base += "/"
-			}
+		// obtain path from environment variable, check to make sure local archive is mounted
+		archiveBase = GetLocalArchiveFolder("pubmed", "Archive", true)
+
+		if archiveBase == "" {
+			DisplayError("Unable to get local archive path")
+			os.Exit(1)
 		}
 
-		postingsBase = base + "Postings"
-		archiveBase = base + "Archive"
-
-		// check to make sure local archive is mounted
-		_, err := os.Stat(archiveBase)
-		if err != nil && os.IsNotExist(err) {
-			fmt.Fprintf(os.Stderr, "\nERROR: Local archive and search index is not mounted\n\n")
+		// obtain path from environment variable, check to make sure local postings directory is mounted
+		if GetLocalArchiveFolder("pubmed", "Postings", true) == "" {
+			DisplayError("Unable to get local postings path")
 			os.Exit(1)
 		}
+
+	}
+
+	errataQuery := "comment on [PROP] OR erratum for [PROP] OR expression of concern for [PROP] OR reprint of [PROP] OR republished in [PROP] OR retracted and republished in [PROP] OR retraction in [PROP] OR update in [PROP]"
+
+	toExclude := ProcessQuery("pubmed", errataQuery, false, false, false, deStop)
+
+	isExcluded := make(map[string]bool)
+	if isExcluded != nil {
+		for _, item := range toExclude {
+			idx := strconv.Itoa(int(item))
+			isExcluded[idx] = true
+		}
 	}
 
 	// gbCitMatch reads partitioned XML from channel and looks up candidate PMIDs
@@ -1577,9 +1586,7 @@ func CreateCitMatchers(inp <-chan XMLRecord, options []string, deStop, doStem bo
 			})
 
 			for _, item := range words {
-				if IsStopWord(item) {
-					continue
-				}
+				// continuing if IsStopWord i or ii here prevented correct matching of PMIDs 4411932 and 4747603 (neurotoxins I and II)
 				temp[item] = true
 			}
 
@@ -1591,7 +1598,7 @@ func CreateCitMatchers(inp <-chan XMLRecord, options []string, deStop, doStem bo
 				}
 			}
 
-			sort.Slice(arry, func(i, j int) bool { return arry[i] < arry[j] })
+			slices.Sort(arry)
 
 			return arry
 		}
@@ -1683,7 +1690,7 @@ func CreateCitMatchers(inp <-chan XMLRecord, options []string, deStop, doStem bo
 				return 0
 			}
 
-			titl = CleanTitle(titl)
+			titl = NormalizeTitle(titl)
 			titl = strings.ToLower(titl)
 			titleWords := sortedWords(titl)
 			titleWords = uniqueWords(titleWords)
@@ -1698,7 +1705,7 @@ func CreateCitMatchers(inp <-chan XMLRecord, options []string, deStop, doStem bo
 			for _, uid := range ids {
 				ttl := getTitle(uid)
 				if ttl != "" {
-					ttl = CleanTitle(ttl)
+					ttl = NormalizeTitle(ttl)
 					ttl = strings.ToLower(ttl)
 					ttlWords := sortedWords(ttl)
 					ttlWords = uniqueWords(ttlWords)
@@ -1745,6 +1752,24 @@ func CreateCitMatchers(inp <-chan XMLRecord, options []string, deStop, doStem bo
 			return res
 		}
 
+		excludeMatches := func(a, b []int32) []int32 {
+
+			temp := make(map[int32]bool)
+			var res []int32
+
+			for _, item := range b {
+				temp[item] = true
+			}
+
+			for _, item := range a {
+				if !temp[item] {
+					res = append(res, item)
+				}
+			}
+
+			return res
+		}
+
 		wordPairs := func(titl string) []string {
 
 			var arry []string
@@ -1846,7 +1871,7 @@ func CreateCitMatchers(inp <-chan XMLRecord, options []string, deStop, doStem bo
 			}
 		*/
 
-		searchTitleParts := func(terms []string, field string) []int32 {
+		searchTitleParts := func(terms []string, field string, min int) []int32 {
 
 			var candidates []int32
 
@@ -1858,7 +1883,7 @@ func CreateCitMatchers(inp <-chan XMLRecord, options []string, deStop, doStem bo
 			pmatch := make(map[int32]int)
 
 			for _, item := range terms {
-				arry := ProcessQuery(postingsBase, "pubmed", item+" ["+field+"]", false, false, false, false, deStop)
+				arry := ProcessQuery("pubmed", item+" ["+field+"]", false, false, false, deStop)
 				for _, uid := range arry {
 					val := pmatch[uid]
 					val++
@@ -1874,8 +1899,8 @@ func CreateCitMatchers(inp <-chan XMLRecord, options []string, deStop, doStem bo
 				}
 			}
 
-			// require at least 4 matching pairs to avoid false positives
-			if max < 4 {
+			// require at least 3 matching pairs or 4 matching words to avoid false positives
+			if max < min {
 				return candidates
 			}
 
@@ -1928,7 +1953,7 @@ func CreateCitMatchers(inp <-chan XMLRecord, options []string, deStop, doStem bo
 
 			var byTitle []int32
 
-			titl = CleanTitle(titl)
+			titl = NormalizeTitle(titl)
 			titl = strings.ToLower(titl)
 			if debug {
 				fmt.Fprintf(os.Stderr, "title:   %s\n", titl)
@@ -1944,7 +1969,7 @@ func CreateCitMatchers(inp <-chan XMLRecord, options []string, deStop, doStem bo
 				return byTitle, "title has no matching word pairs in search index"
 			}
 
-			byTitle = searchTitleParts(pairs, "PAIR")
+			byTitle = searchTitleParts(pairs, "PAIR", 3)
 			if len(byTitle) < 1 {
 				if slower {
 					words := singleWords(titl)
@@ -1952,7 +1977,7 @@ func CreateCitMatchers(inp <-chan XMLRecord, options []string, deStop, doStem bo
 						return byTitle, "title has no matching words in search index"
 					}
 
-					byTitle = searchTitleParts(words, "TITL")
+					byTitle = searchTitleParts(words, "TITL", 4)
 				}
 				if len(byTitle) < 1 {
 					return byTitle, "no candidates for matching title"
@@ -1966,10 +1991,10 @@ func CreateCitMatchers(inp <-chan XMLRecord, options []string, deStop, doStem bo
 
 			var byAuthor []int32
 
-			faut = CleanAuthor(faut)
+			faut = NormalizeAuthor(faut)
 			faut = strings.ToLower(faut)
 
-			laut = CleanAuthor(laut)
+			laut = NormalizeAuthor(laut)
 			laut = strings.ToLower(laut)
 
 			if faut == "" && laut == "" && csrt == "" {
@@ -1979,7 +2004,7 @@ func CreateCitMatchers(inp <-chan XMLRecord, options []string, deStop, doStem bo
 			if faut == "" && laut == "" {
 
 				// no authors, just consortium
-				csrt = CleanTitle(csrt)
+				csrt = NormalizeTitle(csrt)
 				csrt = strings.ToLower(csrt)
 
 				query := csrt + " [CSRT]"
@@ -1989,7 +2014,7 @@ func CreateCitMatchers(inp <-chan XMLRecord, options []string, deStop, doStem bo
 				}
 
 				// find PMIDs indexed under consortium
-				byAuthor = ProcessQuery(postingsBase, "pubmed", query, false, false, false, false, deStop)
+				byAuthor = ProcessQuery("pubmed", query, false, false, false, deStop)
 				if len(byAuthor) < 1 {
 					return byAuthor, "unrecognized consortium '" + csrt + "'", csrt
 				}
@@ -2030,7 +2055,7 @@ func CreateCitMatchers(inp <-chan XMLRecord, options []string, deStop, doStem bo
 			}
 
 			// find PMIDs indexed under first or last author, use wildcard after truncating to single initial
-			byAuthor = ProcessQuery(postingsBase, "pubmed", query, false, false, false, false, deStop)
+			byAuthor = ProcessQuery("pubmed", query, false, false, false, deStop)
 			if len(byAuthor) < 1 {
 				return byAuthor, "unrecognized author '" + names + "'", names
 			}
@@ -2042,7 +2067,7 @@ func CreateCitMatchers(inp <-chan XMLRecord, options []string, deStop, doStem bo
 
 			var byJournal []int32
 
-			jour = CleanJournal(jour)
+			jour = NormalizeJournal(jour)
 			jour = strings.ToLower(jour)
 
 			if jour == "" {
@@ -2054,7 +2079,7 @@ func CreateCitMatchers(inp <-chan XMLRecord, options []string, deStop, doStem bo
 				return byJournal, "unmappable journal '" + jour + "'", jour
 			}
 
-			jour = CleanJournal(jta)
+			jour = NormalizeJournal(jta)
 			jour = strings.ToLower(jour)
 
 			if strings.Index(jour, "|") >= 0 {
@@ -2066,7 +2091,7 @@ func CreateCitMatchers(inp <-chan XMLRecord, options []string, deStop, doStem bo
 				fmt.Fprintf(os.Stderr, "journal: %s\n", query)
 			}
 
-			byJournal = ProcessQuery(postingsBase, "pubmed", query, false, false, false, false, deStop)
+			byJournal = ProcessQuery("pubmed", query, false, false, false, deStop)
 			if len(byJournal) < 1 {
 				return byJournal, "unrecognized journal '" + jour + "'", jour
 			}
@@ -2099,7 +2124,7 @@ func CreateCitMatchers(inp <-chan XMLRecord, options []string, deStop, doStem bo
 
 			span := lst + ":" + nxt
 
-			byYear = ProcessQuery(postingsBase, "pubmed", query, false, false, false, false, deStop)
+			byYear = ProcessQuery("pubmed", query, false, false, false, deStop)
 			if len(byYear) < 1 {
 				return byYear, "unrecognized year range '" + span + "'", span
 			}
@@ -2167,6 +2192,14 @@ func CreateCitMatchers(inp <-chan XMLRecord, options []string, deStop, doStem bo
 
 			working := byTitle
 
+			// exclude erratum for, retraction in, etc.
+			if len(working) > 0 {
+				temp := excludeMatches(working, toExclude)
+				if len(temp) > 0 {
+					working = temp
+				}
+			}
+
 			// restrict by author name
 			if len(byAuthor) > 0 {
 				temp := intersectMatches(working, byAuthor)
@@ -2274,15 +2307,15 @@ func CreateCitMatchers(inp <-chan XMLRecord, options []string, deStop, doStem bo
 				lastname := ""
 				initials := ""
 
-				VisitElements(auth, "LastName", func(str string) {
+				VisitElements(auth, "LastName", "", func(str string) {
 					lastname = str
 				})
 
-				VisitElements(auth, "Initials", func(str string) {
+				VisitElements(auth, "Initials", "", func(str string) {
 					initials = str
 				})
 
-				name := CleanAuthor(lastname + " " + initials)
+				name := NormalizeAuthor(lastname + " " + initials)
 				name = strings.ToLower(name)
 
 				athr = append(athr, name)
@@ -2295,33 +2328,33 @@ func CreateCitMatchers(inp <-chan XMLRecord, options []string, deStop, doStem bo
 				refFields["ATHR"] = strings.Join(athr, ", ")
 			}
 
-			VisitElements(pat, "ArticleTitle", func(str string) {
-				str = CleanTitle(str)
+			VisitElements(pat, "ArticleTitle", "", func(str string) {
+				str = NormalizeTitle(str)
 				refFields["TITL"] = strings.ToLower(str)
 			})
 
 			VisitNodes(pat, "Article/Journal", func(jour *XMLNode) {
 
-				VisitElements(jour, "ISOAbbreviation", func(str string) {
-					str = CleanJournal(str)
+				VisitElements(jour, "ISOAbbreviation", "", func(str string) {
+					str = NormalizeJournal(str)
 					refFields["JOUR"] = strings.ToLower(str)
 				})
 
-				VisitElements(jour, "JournalIssue/Volume", func(str string) {
+				VisitElements(jour, "JournalIssue/Volume", "", func(str string) {
 					refFields["VOL"] = strings.ToLower(str)
 				})
 
-				VisitElements(jour, "JournalIssue/Issue", func(str string) {
+				VisitElements(jour, "JournalIssue/Issue", "", func(str string) {
 					refFields["ISS"] = strings.ToLower(str)
 				})
 
-				VisitElements(jour, "PubDate/Year", func(str string) {
+				VisitElements(jour, "PubDate/Year", "", func(str string) {
 					refFields["YEAR"] = strings.ToLower(str)
 				})
 			})
 
-			VisitElements(pat, "Pagination/MedlinePgn", func(str string) {
-				str = CleanPage(str)
+			VisitElements(pat, "Pagination/MedlinePgn", "", func(str string) {
+				str = NormalizePage(str)
 				refFields["PAGE"] = strings.ToLower(str)
 			})
 
@@ -2378,16 +2411,16 @@ func CreateCitMatchers(inp <-chan XMLRecord, options []string, deStop, doStem bo
 			if citFields["FAUT"] != "" && citFields["LAUT"] != "" {
 				// first author in citation must match first author in record,
 				// OR last author in citation must match last author in record
-				if !compareFields("FAUT", true, false, CleanAuthor) && !compareFields("LAUT", true, false, CleanAuthor) {
+				if !compareFields("FAUT", true, false, NormalizeAuthor) && !compareFields("LAUT", true, false, NormalizeAuthor) {
 					return false
 				}
 			} else if citFields["CSRT"] != "" {
 				// consortium in citation must match consortium in record
-				if !compareFields("CSRT", true, true, CleanTitle) {
+				if !compareFields("CSRT", true, true, NormalizeTitle) {
 					return false
 				}
 			}
-			if !compareFields("TITL", true, true, CleanTitle) {
+			if !compareFields("TITL", true, true, NormalizeTitle) {
 				// if failure, check again without non-alphanumeric characters
 				relaxTitle := func(str string) string {
 
@@ -2395,7 +2428,7 @@ func CreateCitMatchers(inp <-chan XMLRecord, options []string, deStop, doStem bo
 						return ""
 					}
 
-					str = CleanTitle(str)
+					str = NormalizeTitle(str)
 					str = strings.ToLower(str)
 
 					str = RelaxString(str)
@@ -2414,7 +2447,7 @@ func CreateCitMatchers(inp <-chan XMLRecord, options []string, deStop, doStem bo
 
 				// compressed keys matched exactly, so continue with tests
 			}
-			if !compareFields("JOUR", true, false, CleanJournal) {
+			if !compareFields("JOUR", true, false, NormalizeJournal) {
 				return false
 			}
 			if !compareFields("VOL", false, false, nil) {
@@ -2423,7 +2456,7 @@ func CreateCitMatchers(inp <-chan XMLRecord, options []string, deStop, doStem bo
 			if !compareFields("ISS", false, false, nil) {
 				return false
 			}
-			if !compareFields("PAGE", false, false, CleanPage) {
+			if !compareFields("PAGE", false, false, NormalizePage) {
 				return false
 			}
 			if !compareFields("YEAR", true, false, nil) {
@@ -2500,8 +2533,13 @@ func CreateCitMatchers(inp <-chan XMLRecord, options []string, deStop, doStem bo
 					refFields := pma2ref(pma)
 
 					if checkCitedPMID(citFields, refFields) {
-						pmid = orig
-						note = "verified"
+
+						if isExcluded != nil && !isExcluded[orig] {
+							pmid = orig
+							note = "verified"
+						} else {
+							note = "excluded"
+						}
 					}
 				}
 
@@ -2512,6 +2550,12 @@ func CreateCitMatchers(inp <-chan XMLRecord, options []string, deStop, doStem bo
 						pmid = strconv.Itoa(int(pid))
 						if verbose {
 							note = nte
+						} else if note == "excluded" {
+							if pmid == orig {
+								note = "confirmed"
+							} else {
+								note = "replaced"
+							}
 						} else {
 							note = "edirect"
 						}
@@ -2529,12 +2573,27 @@ func CreateCitMatchers(inp <-chan XMLRecord, options []string, deStop, doStem bo
 				jsn := cit2json(ident)
 
 				ok := false
-				ok, note, pmid = json2pmid(jsn)
+				nte := ""
+				ok, nte, pmid = json2pmid(jsn)
 
 				if !ok {
 					// try a second time on server failure
 					jsn = cit2json(ident)
-					_, note, pmid = json2pmid(jsn)
+					_, nte, pmid = json2pmid(jsn)
+				}
+
+				if pmid != "" {
+					if verbose {
+						note = nte
+					} else if note == "excluded" {
+						if pmid == orig {
+							note = "accepted"
+						} else {
+							note = "overridden"
+						}
+					} else {
+						note = nte
+					}
 				}
 			}
 
@@ -2600,7 +2659,7 @@ func CreateCitMatchers(inp <-chan XMLRecord, options []string, deStop, doStem bo
 	var wg sync.WaitGroup
 
 	// launch multiple citmatch goroutines
-	for i := 0; i < NumServe(); i++ {
+	for range numServe {
 		wg.Add(1)
 		go gbCitMatch(&wg, inp, out)
 	}
