@@ -309,28 +309,8 @@ func xmlFormatter(rcrd, prnt string, inp <-chan XMLToken, offset int, doXML bool
 
 			// check for xml line explicitly set in argument
 			if xml != "" {
-				xml = strings.TrimSpace(xml)
-				if strings.HasPrefix(xml, "<") {
-					xml = xml[1:]
-				}
-				if strings.HasPrefix(xml, "?") {
-					xml = xml[1:]
-				}
-				if strings.HasPrefix(xml, "xml") {
-					xml = xml[3:]
-				}
-				if strings.HasPrefix(xml, " ") {
-					xml = xml[1:]
-				}
-				if strings.HasSuffix(xml, "?>") {
-					xlen := len(xml)
-					xml = xml[:xlen-2]
-				}
-				xml = strings.TrimSpace(xml)
-
-				buffer.WriteString("<?xml ")
+				xml = CleanXmltype(xml)
 				buffer.WriteString(xml)
-				buffer.WriteString(" ?>")
 			} else {
 				buffer.WriteString("<?xml version=\"1.0\" encoding=\"UTF-8\" ?>")
 			}
@@ -339,25 +319,8 @@ func xmlFormatter(rcrd, prnt string, inp <-chan XMLToken, offset int, doXML bool
 
 			// check for doctype taken from XML file or explicitly set in argument
 			if doctype != "" {
-				doctype = strings.TrimSpace(doctype)
-				if strings.HasPrefix(doctype, "<") {
-					doctype = doctype[1:]
-				}
-				if strings.HasPrefix(doctype, "!") {
-					doctype = doctype[1:]
-				}
-				if strings.HasPrefix(doctype, "DOCTYPE") {
-					doctype = doctype[7:]
-				}
-				if strings.HasPrefix(doctype, " ") {
-					doctype = doctype[1:]
-				}
-				doctype = strings.TrimSuffix(doctype, ">")
-				doctype = strings.TrimSpace(doctype)
-
-				buffer.WriteString("<!DOCTYPE ")
+				doctype = CleanDoctype(doctype)
 				buffer.WriteString(doctype)
-				buffer.WriteString(">")
 			} else {
 				buffer.WriteString("<!DOCTYPE ")
 				buffer.WriteString(parent)
