@@ -32,11 +32,12 @@ package eutils
 
 import (
 	"bufio"
-	"fmt"
 	"html"
 	"io"
 	"os"
 	"strings"
+	"sync"
+	"unicode"
 )
 
 // GenBankConverter reads flatfiles and sends INSDSeq XML records down a channel
@@ -48,7 +49,7 @@ func GenBankConverter(inp io.Reader) <-chan string {
 
 	out := make(chan string, chanDepth)
 	if out == nil {
-		fmt.Fprintf(os.Stderr, "Unable to create GenBank converter channel\n")
+		DisplayError("Unable to create GenBank converter channel")
 		os.Exit(1)
 	}
 
@@ -76,6 +77,15 @@ func GenBankConverter(inp io.Reader) <-chan string {
 				if line == "" {
 					continue
 				}
+
+				// https://stackoverflow.com/questions/58994146/how-to-remove-non-printable-characters
+				line = strings.Map(func(r rune) rune {
+					if unicode.IsPrint(r) {
+						return r
+					}
+					return -1
+				}, line)
+
 				return line
 			}
 			return ""
@@ -193,7 +203,7 @@ func GenBankConverter(inp io.Reader) <-chan string {
 					writeOneElement("    ", "INSDSeq_update-date", cols[6])
 
 				} else {
-					fmt.Fprintf(os.Stderr, "ERROR: "+str+"\n")
+					DisplayWarning("%s", str)
 				}
 
 				// read next line and continue - handled by readContinuationLines above
@@ -230,7 +240,7 @@ func GenBankConverter(inp io.Reader) <-chan string {
 					writeOneElement("    ", "INSDSeq_primary-accession", accessions[0])
 
 				} else {
-					fmt.Fprintf(os.Stderr, "ERROR: ACCESSION "+str+"\n")
+					DisplayError("ACCESSION %s", str)
 				}
 			}
 
@@ -256,7 +266,7 @@ func GenBankConverter(inp io.Reader) <-chan string {
 					}
 
 				} else {
-					fmt.Fprintf(os.Stderr, "ERROR: "+line+"\n")
+					DisplayError("%s", line)
 				}
 
 				// read next line and continue
@@ -397,7 +407,7 @@ func GenBankConverter(inp io.Reader) <-chan string {
 							posit := strings.Join(arry, ",")
 							writeOneElement("        ", "INSDReference_position", posit)
 						} else {
-							fmt.Fprintf(os.Stderr, "ERROR: "+posn+"\n")
+							DisplayError("%s", posn)
 						}
 					}
 				} else {
@@ -513,7 +523,7 @@ func GenBankConverter(inp io.Reader) <-chan string {
 						break
 					}
 					if len(line) < 22 {
-						fmt.Fprintf(os.Stderr, "ERROR: "+line+"\n")
+						DisplayError("%s", line)
 						line = nextLine()
 						row++
 						continue
@@ -718,7 +728,28 @@ func GenBankConverter(inp io.Reader) <-chan string {
 						rec.WriteString("        <INSDFeature_partial3 value=\"true\"/>\n")
 					}
 
+					isLegalQualifier := func(str string) bool {
+
+						if str == "" {
+							return false
+						}
+
+						lft, rgt := SplitInTwoLeft(str, "=")
+						if lft == "" || rgt == "" {
+							return false
+						}
+
+						ky := strings.ToUpper(lft)
+						val, ok := legalQuals[ky]
+						if ok && lft == val {
+							return true
+						}
+
+						return false
+					}
+
 					hasQual := false
+					hasMolWt := false
 					for {
 						if !strings.HasPrefix(line, twentyonespaces) {
 							// if not qualifier line, break out of loop
@@ -748,7 +779,7 @@ func GenBankConverter(inp io.Reader) <-chan string {
 									break
 								}
 								txt := strings.TrimPrefix(line, twentyonespaces)
-								if strings.HasPrefix(txt, "/") {
+								if strings.HasPrefix(txt, "/") && isLegalQualifier(txt[1:]) {
 									// if not continuation of qualifier, break out of loop
 									break
 								}
@@ -760,13 +791,33 @@ func GenBankConverter(inp io.Reader) <-chan string {
 								}
 							}
 
-							rec.WriteString("          <INSDQualifier>\n")
+							if qual == "country" && fkey == "source" {
+								qual = "geo_loc_name"
+							}
 
-							writeOneElement("            ", "INSDQualifier_name", qual)
+							if qual == "calculated_mol_wt" || qual == "mol_wt" {
+								hasMolWt = true
+							}
 
 							val = strings.TrimPrefix(val, "\"")
 							val = strings.TrimSuffix(val, "\"")
 							val = strings.TrimSpace(val)
+
+							if qual == "translation" && !hasMolWt {
+								mw := ProteinWeight(val, true, false)
+								if mw != "" {
+									rec.WriteString("          <INSDQualifier>\n")
+									writeOneElement("            ", "INSDQualifier_name", "calculated_mol_wt")
+									writeOneElement("            ", "INSDQualifier_value", mw)
+									rec.WriteString("          </INSDQualifier>\n")
+									hasMolWt = true
+								}
+							}
+
+							rec.WriteString("          <INSDQualifier>\n")
+
+							writeOneElement("            ", "INSDQualifier_name", qual)
+
 							if val != "" {
 
 								writeOneElement("            ", "INSDQualifier_value", val)
@@ -946,7 +997,7 @@ func GenBankConverter(inp io.Reader) <-chan string {
 
 				cols := strings.Fields(line)
 				if len(cols) > 0 && !IsAllDigits(cols[0]) {
-					fmt.Fprintf(os.Stderr, "ERROR: Unrecognized section "+cols[0]+"\n")
+					DisplayError("Unrecognized section %s", cols[0])
 				}
 
 				for _, str := range cols {
@@ -975,21 +1026,26 @@ func GenBankConverter(inp io.Reader) <-chan string {
 	return out
 }
 
-/*
+// GenBanktoXML converts a GenBank flatfile to an XML string
+func GenBanktoXML(gbff string) string {
+
+	return StringToXML(gbff, GenBankConverter)
+}
+
 // CreateGBConverters makes concurrent calls to GenBankConverter
-func CreateGBConverters(inp <-chan string) <-chan string {
+func CreateGBConverters(inp <-chan TextRecord) <-chan TextRecord {
 
 	if inp == nil {
 		return nil
 	}
 
-	out := make(chan string, ChanDepth())
+	out := make(chan TextRecord, chanDepth)
 	if out == nil {
-		fmt.Fprintf(os.Stderr, "\nERROR: Unable to create gb converter channel\n")
+		DisplayError("Unable to create gb converter channel")
 		os.Exit(1)
 	}
 
-	gbConvert := func(wg *sync.WaitGroup, inp <-chan string, out chan<- string) {
+	gbConvert := func(wg *sync.WaitGroup, inp <-chan TextRecord, out chan<- TextRecord) {
 
 		// report when this matcher has no more records to process
 		defer wg.Done()
@@ -1000,7 +1056,7 @@ func CreateGBConverters(inp <-chan string) <-chan string {
 
 			gbcq := GenBankConverter(in)
 			if out == nil {
-				fmt.Fprintf(os.Stderr, "\nERROR: Unable to create gb converter channel\n")
+				DisplayError("Unable to create gb converter channel")
 				os.Exit(1)
 			}
 
@@ -1016,18 +1072,18 @@ func CreateGBConverters(inp <-chan string) <-chan string {
 		}
 
 		// read partitioned XML from producer channel
-		for str := range inp {
+		for curr := range inp {
 
-			txt := convertOneGB(str)
+			txt := convertOneGB(curr.Text)
 
-			out <- txt
+			out <- TextRecord{Index: curr.Index, Text: txt}
 		}
 	}
 
 	var wg sync.WaitGroup
 
 	// launch multiple gb converter goroutines
-	for i := 0; i < NumServe(); i++ {
+	for range numServe {
 		wg.Add(1)
 		go gbConvert(&wg, inp, out)
 	}
@@ -1040,4 +1096,3 @@ func CreateGBConverters(inp <-chan string) <-chan string {
 
 	return out
 }
-*/
