@@ -32,8 +32,9 @@ package eutils
 
 import (
 	"fmt"
+	"maps"
 	"os"
-	"sort"
+	"slices"
 	"strings"
 )
 
@@ -99,6 +100,14 @@ var ncbieaaCode = map[int]string{
 	33: "FFLLSSSSYYY*CCWWLLLLPPPPHHQQRRRRIIIMTTTTNNKKSSSKVVVVAAAADDEEGGGG",
 }
 
+// Alternative start codons may encode Leucine, Isoleucine, or Valine during
+// elongation, but a separate initiator tRNA always inserts Methionine (or
+// formyl-Methionine (fMet) in bacteria) the start of translation.
+
+// The sncbieaaCode field was expanded to include potential translation
+// termination codons, indicated by asterisks, upon discovery of ciliate
+// genetic codes without dedicated stop codons.
+
 // Base
 //    1  TTTTTTTTTTTTTTTTCCCCCCCCCCCCCCCCAAAAAAAAAAAAAAAAGGGGGGGGGGGGGGGG
 //    2  TTTTCCCCAAAAGGGGTTTTCCCCAAAAGGGGTTTTCCCCAAAAGGGGTTTTCCCCAAAAGGGG
@@ -380,8 +389,11 @@ func TranslateCdRegion(seq string, genCode, frame int, includeStop, doEveryCodon
 	// start_state := 0
 	firstTime := true
 
-	// standard and bacterial code use built-in array for fast access
-	fast := genCode == 1 || genCode == 11
+	// look up fully-resolved precomputed genetic code array
+	genCodeTable, ok := genCodeMaps[genCode]
+	if !ok {
+		genCodeTable = genCodeMaps[1]
+	}
 
 	pos := frame
 
@@ -391,7 +403,7 @@ func TranslateCdRegion(seq string, genCode, frame int, includeStop, doEveryCodon
 	if length > 0 {
 
 		// loop through first codon to initialize state
-		for k := 0; k < 3; k++ {
+		for range 3 {
 
 			// internal version of NextCodonState without value reality check
 			ch := int(seq[pos])
@@ -418,7 +430,7 @@ func TranslateCdRegion(seq string, genCode, frame int, includeStop, doEveryCodon
 	for i := 1; i < length; i++ {
 
 		// loop through next triplet codon
-		for k := 0; k < 3; k++ {
+		for range 3 {
 
 			// internal version of NextCodonState without value reality check
 			ch := int(seq[pos])
@@ -428,12 +440,8 @@ func TranslateCdRegion(seq string, genCode, frame int, includeStop, doEveryCodon
 		}
 
 		// save translated amino acid
-		if fast {
-			// use high-efficiency built-in lookup table for standard code
-			aa = int(stdGenCode[state])
-		} else {
-			aa = GetCodonResidue(genCode, state)
-		}
+		aa = int(genCodeTable[state])
+		// aa = GetCodonResidue(genCode, state)
 
 		aminos = append(aminos, string(rune(aa)))
 
@@ -518,6 +526,138 @@ func TranslateCdRegion(seq string, genCode, frame int, includeStop, doEveryCodon
 	return txt
 }
 
+// TranslateAllFrames produces six-frame translation in a single pass
+func TranslateAllFrames(seq string, genCode int, circular, orf bool) []string {
+
+	var arry []string
+
+	if len(seq) < 6 {
+		return nil
+	}
+
+	// strings.Builder.WriteByte is inlined by the compiler
+	var forward [3]strings.Builder
+	var reverse [3]strings.Builder
+
+	genCode = correctGenCode(genCode)
+
+	// look up fully-resolved precomputed genetic code array
+	genCodeTable, ok := genCodeMaps[genCode]
+	if !ok {
+		genCodeTable = genCodeMaps[1]
+	}
+
+	if orf {
+		// orf option flags start codon states with upper-case amino acid letters
+		gc, ok := orfStartMaps[genCode]
+		if ok {
+			tmp := []byte(strings.ToLower(genCodeTable))
+			// single variable range on map gives index (state), not value (amino acid)
+			for state := range gc {
+				tmp[state] = genCodeTable[state]
+			}
+			// replace original with case-modified version
+			genCodeTable = string(tmp)
+		}
+	}
+
+	state := 0
+
+	// prime state table with first two nucleotides
+	for _, ch := range seq[:2] {
+		state = nextState[state] + baseToIdx[int(ch)]
+	}
+
+	frame := 0
+
+	// sending a slice does not copy the original data
+	translateThreeFrames := func(subseq string) {
+
+		// use local copies of state and frame for speed
+		st := state
+		fr := frame
+
+		for _, ch := range subseq {
+
+			// codon to residue for plus strand
+			st = nextState[st] + baseToIdx[int(ch)]
+			forward[fr].WriteByte(genCodeTable[st])
+
+			// reverse-complemented codon to residue for minus strand
+			rv := rvCpState[st]
+			reverse[fr].WriteByte(genCodeTable[rv])
+
+			// cycle through three reading frames on each strand
+			fr++
+			if fr > 2 {
+				// reset frame
+				fr = 0
+			}
+		}
+
+		// update state and frame closure variables
+		state = st
+		frame = fr
+	}
+
+	// process remainder of sequence through finite state machine
+	translateThreeFrames(seq[2:])
+
+	trunc := 0
+
+	if circular {
+
+		// feed in first two bases used to prime state machine
+		translateThreeFrames(seq[:2])
+
+		// calculate final length
+		for fr := range 3 {
+			ln := forward[fr].Len()
+			if ln > trunc {
+				trunc = ln
+			}
+		}
+
+		// translate two more bases, then truncate all to final length
+		translateThreeFrames(seq[2:4])
+
+		// for 10-base circular molecule, third frame needs two additional
+		// bases beyond the primer to generate all possible codons
+		// ATG TTT CTG A|AT G
+		// TGT TTC TGA| ATG
+		// GTT TCT GA|A TGT
+	}
+
+	// write translations from plus strand codons in all three frames
+	for frame := range 3 {
+		tmp := forward[frame].String()
+		if trunc > 0 && len(tmp) > trunc {
+			arry = append(arry, tmp[:trunc])
+		} else {
+			arry = append(arry, tmp[:])
+		}
+	}
+
+	// need to reverse the order of residues translated on minus strand from
+	// reverse-complemented codons but proceeding in 3' to 5' direction
+	for frame := range 3 {
+		tmp := reverse[frame].String()
+		if trunc > 0 && len(tmp) > trunc {
+			arry = append(arry, ReverseString(tmp[:trunc]))
+		} else {
+			arry = append(arry, ReverseString(tmp[:]))
+		}
+	}
+
+	/*
+		combining circular and orf flags should provide enough information to allow
+		reconstruction of coding region spanning origin of circular molecules, such
+		as plasmids, mitochondria, or (most) bacterial chromosomes
+	*/
+
+	return arry
+}
+
 // NucProtCodonReport displays triplet codons above the translated amino acid
 func NucProtCodonReport(nuc, prt string, frame int, threeLetter bool) string {
 
@@ -534,7 +674,7 @@ func NucProtCodonReport(nuc, prt string, frame int, threeLetter bool) string {
 
 	var codons []string
 
-	for i := 0; i < length; i++ {
+	for range length {
 		pos += 3
 		codons = append(codons, nuc[lag:pos])
 		lag = pos
@@ -602,11 +742,7 @@ func NucProtCodonReport(nuc, prt string, frame int, threeLetter bool) string {
 // PrintGeneticCodeTables prints a tab-delimited table of all genetic codes
 func PrintGeneticCodeTables() {
 
-	var keys []int
-	for ky := range ncbieaaCode {
-		keys = append(keys, ky)
-	}
-	sort.Slice(keys, func(i, j int) bool { return keys[i] < keys[j] })
+	keys := slices.Sorted(maps.Keys(ncbieaaCode))
 
 	for _, id := range keys {
 		fmt.Fprintf(os.Stdout, "%d\t%s\t%s\t%s\n", id, ncbieaaCode[id], sncbieaaCode[id], genCodeNames[id])
@@ -659,7 +795,7 @@ func GenerateGeneticCodeMaps() {
 	lowerToBase := "-acmgrsvtwyhkdbn"
 
 	// illegal characters map to 0
-	for i := 0; i < 256; i++ {
+	for i := range 256 {
 		baseToIdx[i] = baseGap
 	}
 
@@ -680,19 +816,14 @@ func GenerateGeneticCodeMaps() {
 		baseToIdx[i] = i
 	}
 
-	spaces := ""
-	fmt.Fprintf(os.Stdout, "var baseToIdx = map[int]int{\n")
-	for i := 0; i < 256; i++ {
-		if i < 10 {
-			spaces = "  "
-		} else if i < 100 {
-			spaces = " "
-		} else {
-			spaces = ""
-		}
-		if baseToIdx[i] != 0 {
-			fmt.Fprintf(os.Stdout, "\t%d:%s %d,\n", i, spaces, baseToIdx[i])
+	fmt.Fprintf(os.Stdout, "var baseToIdx = [256]int{\n")
+	for i := 0; i < 256; i += 16 {
+		prefix := "\t"
+		for j := range 16 {
+			fmt.Fprintf(os.Stdout, "%s%4d,", prefix, baseToIdx[i+j])
+			prefix = " "
 		}
+		fmt.Fprintf(os.Stdout, "\n")
 	}
 	fmt.Fprintf(os.Stdout, "}\n\n")
 
@@ -722,7 +853,7 @@ func GenerateGeneticCodeMaps() {
 	fmt.Fprintf(os.Stdout, "var nextState = [4096]int{\n")
 	for i := 0; i < 4096; i += 16 {
 		prefix := "\t"
-		for j := 0; j < 16; j++ {
+		for j := range 16 {
 			fmt.Fprintf(os.Stdout, "%s%4d,", prefix, nextState[i+j])
 			prefix = " "
 		}
@@ -734,7 +865,7 @@ func GenerateGeneticCodeMaps() {
 	fmt.Fprintf(os.Stdout, "var rvCpState = [4096]int{\n")
 	for i := 0; i < 4096; i += 16 {
 		prefix := "\t"
-		for j := 0; j < 16; j++ {
+		for j := range 16 {
 			fmt.Fprintf(os.Stdout, "%s%4d,", prefix, rvCpState[i+j])
 			prefix = " "
 		}
@@ -754,7 +885,7 @@ func GenerateGeneticCodeMaps() {
 
 		tbl := new(TransTable)
 		if tbl == nil {
-			fmt.Fprintf(os.Stderr, "\nERROR: Unable to allocate translation table for genetic code %d\n", genCode)
+			DisplayError("Unable to allocate translation table for genetic code %d", genCode)
 			os.Exit(1)
 		}
 
@@ -770,23 +901,23 @@ func GenerateGeneticCodeMaps() {
 		// return if unable to find ncbieaa and sncbieaa strings
 		ncbieaa, ok := ncbieaaCode[genCode]
 		if !ok {
-			fmt.Fprintf(os.Stderr, "\nERROR: Genetic code %d does not exist\n", genCode)
+			DisplayError("Genetic code %d does not exist", genCode)
 			os.Exit(1)
 		}
 		sncbieaa, ok := sncbieaaCode[genCode]
 		if !ok {
-			fmt.Fprintf(os.Stderr, "\nERROR: Genetic code %d does not exist\n", genCode)
+			DisplayError("Genetic code %d does not exist", genCode)
 			os.Exit(1)
 		}
 
 		// also check length of ncbieaa and sncbieaa strings
 		if len(ncbieaa) != 64 || len(sncbieaa) != 64 {
-			fmt.Fprintf(os.Stderr, "\nERROR: Genetic code %d length mismatch\n", genCode)
+			DisplayError("Genetic code %d length mismatch", genCode)
 			os.Exit(1)
 		}
 
 		// ambiguous codons map to unknown amino acid or not start
-		for i := 0; i < 4096; i++ {
+		for i := range 4096 {
 			tbl.aminoAcid[i] = int('X')
 			tbl.orfStart[i] = int('-')
 			tbl.orfStop[i] = int('-')
@@ -881,15 +1012,11 @@ func GenerateGeneticCodeMaps() {
 	// generate aminoAcidMaps, orfStartMaps, and orfStopMaps for source code
 
 	// sort genetic code keys in numerical order
-	var keys []int
-	for ky := range sncbieaaCode {
-		keys = append(keys, ky)
-	}
-	sort.Slice(keys, func(i, j int) bool { return keys[i] < keys[j] })
+	keys := slices.Sorted(maps.Keys(sncbieaaCode))
 
 	tbl := newTransTable(1)
 	if tbl == nil {
-		fmt.Fprintf(os.Stderr, "newTransTable failed\n")
+		DisplayError("newTransTable failed")
 		return
 	}
 
@@ -897,9 +1024,9 @@ func GenerateGeneticCodeMaps() {
 	fmt.Fprintf(os.Stdout, "// remaining tables show differences from the standard\n\n")
 	fmt.Fprintf(os.Stdout, "var aminoAcidMaps = map[int]map[int]int{\n")
 
-	spaces = ""
+	spaces := ""
 	fmt.Fprintf(os.Stdout, "\t1: {\n")
-	for i := 0; i < 4096; i++ {
+	for i := range 4096 {
 		aa := tbl.aminoAcid[i]
 		if aa != 0 && aa != 'X' {
 			if i < 10 {
@@ -925,13 +1052,13 @@ func GenerateGeneticCodeMaps() {
 
 		tb := newTransTable(id)
 		if tb == nil {
-			fmt.Fprintf(os.Stderr, "newTransTable failed for code %d, skipping\n", id)
+			DisplayWarning("newTransTable failed for code %d, skipping", id)
 			continue
 		}
 
 		spaces = ""
 		fmt.Fprintf(os.Stdout, "\t%d: {\n", id)
-		for i := 0; i < 4096; i++ {
+		for i := range 4096 {
 			aa := tbl.aminoAcid[i]
 			nw := tb.aminoAcid[i]
 			if aa == nw {
@@ -965,13 +1092,13 @@ func GenerateGeneticCodeMaps() {
 
 		tb := newTransTable(id)
 		if tb == nil {
-			fmt.Fprintf(os.Stderr, "newTransTable failed for code %d, skipping\n", id)
+			DisplayWarning("newTransTable failed for code %d, skipping", id)
 			continue
 		}
 
 		spaces = ""
 		fmt.Fprintf(os.Stdout, "\t%d: {\n", id)
-		for i := 0; i < 4096; i++ {
+		for i := range 4096 {
 			aa := tb.orfStart[i]
 			if aa != 0 && aa != '*' && aa != '-' && aa != 'X' {
 				if i < 10 {
@@ -997,13 +1124,13 @@ func GenerateGeneticCodeMaps() {
 
 		tb := newTransTable(id)
 		if tb == nil {
-			fmt.Fprintf(os.Stderr, "newTransTable failed for code %d, skipping\n", id)
+			DisplayWarning("newTransTable failed for code %d, skipping", id)
 			continue
 		}
 
 		spaces = ""
 		fmt.Fprintf(os.Stdout, "\t%d: {\n", id)
-		for i := 0; i < 4096; i++ {
+		for i := range 4096 {
 			aa := tb.orfStop[i]
 			if aa == '*' {
 				if i < 10 {
@@ -1023,24 +1150,44 @@ func GenerateGeneticCodeMaps() {
 
 	fmt.Fprintf(os.Stdout, "}\n\n")
 
-	// generate static string of standard genetic code array for performance
+	// finish with instantiated genetic code arrays
 
-	fmt.Fprintf(os.Stdout, "// standard genetic code array for performance\n\n")
-	fmt.Fprintf(os.Stdout, "const stdGenCode = \"\" +\n")
+	fmt.Fprintf(os.Stdout, "// genetic code arrays for performance for each genetic code\n\n")
+	fmt.Fprintf(os.Stdout, "var genCodeMaps = map[int]string{\n")
 
-	for i := 0; i < 4096; i += 64 {
-		fmt.Fprintf(os.Stdout, "\t\"")
-		for j := 0; j < 64; j++ {
-			aa := tbl.aminoAcid[i+j]
-			fmt.Fprintf(os.Stdout, "%c", aa)
+	for _, id := range keys {
+
+		tb := newTransTable(id)
+		if tb == nil {
+			DisplayWarning("newTransTable failed for code %d, skipping", id)
+			continue
 		}
-		fmt.Fprintf(os.Stdout, "\"")
-		if i < 4032 {
-			fmt.Fprintf(os.Stdout, " +")
+
+		fmt.Fprintf(os.Stdout, "\t%d: ", id)
+		if id < 10 {
+			fmt.Fprintf(os.Stdout, " ")
+		}
+		for i := 0; i < 4096; i += 64 {
+			if i > 0 {
+				fmt.Fprintf(os.Stdout, "\t\t")
+			}
+			fmt.Fprintf(os.Stdout, "\"")
+			for j := range 64 {
+				aa := tb.aminoAcid[i+j]
+				fmt.Fprintf(os.Stdout, "%c", aa)
+			}
+			fmt.Fprintf(os.Stdout, "\"")
+			if i < 4032 {
+				fmt.Fprintf(os.Stdout, " +")
+			} else {
+				fmt.Fprintf(os.Stdout, ",")
+			}
+			fmt.Fprintf(os.Stdout, "\n")
 		}
-		fmt.Fprintf(os.Stdout, "\n")
 	}
 
+	fmt.Fprintf(os.Stdout, "}\n")
+
 	/*
 		// print codon to state table for reference
 
