@@ -36,15 +36,14 @@ import (
 	"fmt"
 	"html"
 	"io"
+	"maps"
 	"os"
 	"path/filepath"
 	"regexp"
 	"runtime"
-	"sort"
-	"strconv"
+	"slices"
 	"strings"
 	"sync"
-	"time"
 	"unicode"
 )
 
@@ -59,13 +58,24 @@ import (
   <IdxDocument>
     <IdxUid>2539356</IdxUid>
     <IdxSearchFields>
+      <UID>02539356</UID>
+      <SIZE>13727</SIZE>
       <YEAR>1989</YEAR>
+      <DATE>1989 04</DATE>
+      <RDAT>2019 05 08</RDAT>
       <JOUR>J Bacteriol</JOUR>
-      <JOUR>Journal of bacteriology</JOUR>
+      <JOUR>2985120R</JOUR>
+      <JOUR>0021-9193</JOUR>
+      <JOUR>Journal of Bacteriology</JOUR>
+      <JOUR>J Bacteriol</JOUR>
+      <JOUR>0021-9193</JOUR>
       <VOL>171</VOL>
       <ISS>4</ISS>
       <PAGE>1904</PAGE>
-      ...
+      <LANG>eng</LANG>
+      <ANUM>2</ANUM>
+      <FAUT>Kans JA</FAUT>
+      <LAUT>Casadaban MJ</LAUT>
       <AUTH>Kans JA</AUTH>
       <AUTH>Casadaban MJ</AUTH>
       <TITL pos="7">immunity</TITL>
@@ -74,6 +84,7 @@ import (
       <TITL pos="2">sequences</TITL>
       <TITL pos="5">tn3</TITL>
       <TITL pos="6">transposition</TITL>
+      <TIAB pos="145">38</TIAB>
       <TIAB pos="126">acting</TIAB>
       <TIAB pos="188">additional</TIAB>
       <TIAB pos="146">base</TIAB>
@@ -84,10 +95,14 @@ import (
       <PAIR>sequences required</PAIR>
       <PAIR>tn3 transposition</PAIR>
       <PAIR>transposition immunity</PAIR>
+      <PTYP>Journal Article</PTYP>
+      <PTYP>Research Support, U.S. Gov&#39;t, P.H.S.</PTYP>
       <PROP>Published In Print</PROP>
-      <PROP>Journal Article</PROP>
-      <PROP>Research Support, U.S. Gov&#39;t, P.H.S.</PROP>
       <PROP>Has Abstract</PROP>
+      <DOI>9891 4191 4091 4 171 bj 8211 01</DOI>
+      <PMCID>209839</PMCID>
+      <CODE>d001483</CODE>
+      <CODE>d002874</CODE>
       ...
       <MESH>Plasmids</MESH>
       <MESH>Recombination, Genetic</MESH>
@@ -138,156 +153,50 @@ import (
 // ENTREZ2INDEX COMMAND GENERATOR
 
 // MakeE2Commands generates extraction commands to create input for Entrez2Index
-func MakeE2Commands(tform, db string, isPipe bool) []string {
-
-	currentYear := strconv.Itoa(time.Now().Year())
+func MakeE2Commands(tform, idxargs string) []string {
 
 	var acc []string
 
-	if !isPipe {
-		if !deStop {
-			acc = append(acc, "-stops")
-		}
-		if doStem {
-			acc = append(acc, "-stems")
-		}
+	// idxargs file contains one command or argument per line
+	if idxargs == "" {
+		return acc
 	}
 
-	if db == "" || db == "pubmed" {
+	inFile, err := os.Open(idxargs)
+	if err != nil {
+		DisplayError("Unable to open index argument array file: %s\n", err.Error())
+		return acc
+	}
+	defer inFile.Close()
 
-		acc = append(acc, "-set", "IdxDocumentSet", "-rec", "IdxDocument")
-		acc = append(acc, "-pattern", "PubmedArticle", "-UID", "MedlineCitation/PMID")
-		acc = append(acc, "-wrp", "IdxUid", "-element", "&UID", "-clr", "-rst", "-tab", "")
+	scanr := bufio.NewScanner(inFile)
+	if scanr == nil {
+		DisplayError("Unable to create NewScanner")
+		return acc
+	}
 
-		acc = append(acc, "-group", "PubmedArticle", "-pkg", "IdxSearchFields")
+	for scanr.Scan() {
 
-		// identifier field - UID
+		line := scanr.Text()
 
-		acc = append(acc, "-block", "PubmedArticle", "-wrp", "UID", "-pad", "&UID")
+		// do NOT skip empty line or trim spaces, to allow "" or " " arguments
 
-		// date fields - YEAR, PDAT, and RDAT
-
-		acc = append(acc, "-block", "PubmedArticle", "-wrp", "YEAR", "-year", "PubDate/*")
-		acc = append(acc, "-block", "PubmedArticle", "-unit", "PubDate", "-wrp", "PDAT", "-reg", "/", "-exp", " ", "-date", "*")
-		acc = append(acc, "-block", "PubmedArticle", "-unit", "DateRevised", "-wrp", "RDAT", "-reg", "/", "-exp", " ", "-date", "*")
-
-		// citation fields - JOUR, VOL, ISS, PAGE, and LANG
-
-		acc = append(acc, "-block", "MedlineJournalInfo", "-wrp", "JOUR", "-element", "MedlineTA", "NlmUniqueID", "ISSNLinking")
-		acc = append(acc, "-block", "Article/Journal", "-wrp", "JOUR", "-jour", "Title", "ISOAbbreviation", "-element", "ISSN")
-		acc = append(acc, "-wrp", "VOL", "-element", "Volume", "-wrp", "ISS", "-element", "Issue")
-		acc = append(acc, "-block", "Article/Pagination", "-wrp", "PAGE", "-page", "MedlinePgn")
-		acc = append(acc, "-block", "Article/Language", "-wrp", "LANG", "-element", "Language")
-
-		// author fields - ANUM, AUTH, FAUT, LAUT, CSRT, INUM, and INVR
-
-		// only count human authors, not consortia
-		acc = append(acc, "-block", "AuthorList", "-wrp", "ANUM", "-num", "Author/LastName")
-		// use -position to get first author
-		acc = append(acc, "-block", "AuthorList/Author", "-position", "first")
-		acc = append(acc, "-wrp", "FAUT", "-sep", " ", "-author", "LastName,Initials")
-		// expect consortium to be last in the author list, so explore each author, and if last name is present,
-		// overwrite the LAST variable with the current person's name
-		acc = append(acc, "-block", "AuthorList/Author", "-if", "LastName", "-sep", " ", "-LAST", "LastName,Initials")
-		// then explore on (single instance) PubmedArticle to print one copy of variable containing the last author's name
-		acc = append(acc, "-block", "PubmedArticle", "-if", "&LAST", "-wrp", "LAUT", "-author", "&LAST")
-		// separate field for consortia
-		acc = append(acc, "-block", "AuthorList/Author", "-wrp", "CSRT", "-prose", "CollectiveName")
-		// now get all human authors and investigators
-		acc = append(acc, "-block", "AuthorList/Author", "-wrp", "AUTH", "-sep", " ", "-author", "LastName,Initials")
-		acc = append(acc, "-block", "InvestigatorList/Investigator", "-wrp", "INVR", "-sep", " ", "-author", "LastName,Initials")
-		// optionally index number of investigators
-		// acc = append(acc, "-block", "InvestigatorList", "-wrp", "INUM", "-num", "Investigator/LastName")
-
-		// title and abstract fields - TIAB, TITL, and PAIR
-
-		// positional indices for TITL and TIAB fields
-		acc = append(acc, "-block", "PubmedArticle", "-article", "ArticleTitle")
-		acc = append(acc, "-block", "PubmedArticle", "-indices", "ArticleTitle,Abstract/AbstractText")
-		// overlapping adjacent word pairs (or isolated singletons) separated by stop words
-		acc = append(acc, "-block", "PubmedArticle", "-wrp", "PAIR", "-pairx", "ArticleTitle")
-
-		// property fields - PROP and PTYP
-
-		acc = append(acc, "-block", "PublicationType", "-wrp", "PTYP", "-element", "PublicationType")
-
-		acc = append(acc, "-block", "CommentsCorrections", "-wrp", "PROP", "-prop", "@RefType")
-		acc = append(acc, "-block", "PublicationStatus", "-wrp", "PROP", "-prop", "PublicationStatus")
-		acc = append(acc, "-block", "Abstract", "-if", "AbstractText", "-wrp", "PROP", "-lbl", "Has Abstract")
-		acc = append(acc, "-block", "MedlineCitation", "-if", "CoiStatement", "-wrp", "PROP", "-lbl", "Conflict of Interest Statement")
-		// dates
-		acc = append(acc, "-block", "Journal", "-if", "MedlineDate", "-wrp", "PROP", "-lbl", "Medline Date")
-		acc = append(acc, "-subset", "MedlineDate", "-if", "%MedlineDate", "-lt", "4", "-wrp", "PROP", "-lbl", "Bad Date")
-		acc = append(acc, "-block", "PubDate", "-if", "Year", "-and", "%Year", "-lt", "4", "-wrp", "PROP", "-lbl", "Bad Date")
-		acc = append(acc, "-block", "PubMedPubDate", "-if", "%Year", "-lt", "4", "-wrp", "PROP", "-lbl", "Bad Date")
-		acc = append(acc, "-block", "JournalIssue", "-if", "@CitedMedium", "-is-not", "Internet")
-		acc = append(acc, "-subset", "PubDate", "-if", "Year", "-gt", currentYear, "-wrp", "PROP", "-lbl", "Future Date")
-		acc = append(acc, "-block", "PubMedPubDate", "-if", "Year", "-gt", currentYear, "-and")
-		acc = append(acc, "@PubStatus", "-is-not", "pmc-release", "-wrp", "PROP", "-lbl", "Future Date")
-		// version
-		acc = append(acc, "-block", "MedlineCitation", "-if", "PMID@Version", "-gt", "1", "-wrp", "PROP", "-lbl", "Versioned")
-
-		// optionally index record size to find annotation outliers (e.g., PMID 33766997)
-		// acc = append(acc, "-block", "PubmedArticle", "-wrp", "SIZE", "-len", "*")
-
-		// if Extras/meshtree.txt is available, index CODE, TREE, and SUBH fields, and MESH for term list
-		if tform != "" {
-			acc = append(acc, "-block", "PubmedArticle", "-meshcode")
-			acc = append(acc, "MeshHeading/DescriptorName@UI,Chemical/NameOfSubstance@UI,SupplMeshName@UI")
-			acc = append(acc, "-block", "MeshHeading/QualifierName", "-wrp", "SUBH", "-element", "QualifierName")
-			// only populating MESH for live term list, since query will redirect to wildcard on TREE
-			acc = append(acc, "-block", "MeshHeading/DescriptorName", "-wrp", "MESH", "-element", "DescriptorName")
-		}
-
-	} else if db == "pmc" {
-
-		acc = append(acc, "-set", "IdxDocumentSet", "-rec", "IdxDocument")
-		acc = append(acc, "-pattern", "PMCExtract", "-UID", "PMCExtract/UID")
-		acc = append(acc, "-wrp", "IdxUid", "-element", "&UID", "-clr", "-rst", "-tab", "")
-
-		acc = append(acc, "-group", "PMCExtract", "-pkg", "IdxSearchFields")
-
-		// identifier field - UID
-
-		acc = append(acc, "-block", "PMCExtract", "-wrp", "UID", "-pad", "&UID")
-
-		// date field - YEAR
-
-		acc = append(acc, "-block", "PMCExtract", "-wrp", "YEAR", "-element", "YEAR")
-
-		// citation field - JOUR
-
-		acc = append(acc, "-block", "PMCExtract", "-wrp", "JOUR", "-element", "JOUR")
-		acc = append(acc, "-block", "PMCExtract", "-wrp", "JOUR", "-element", "SRC")
-
-		// author field - AUTH
-
-		acc = append(acc, "-block", "PMCExtract", "-subset", "Auth", "-wrp", "AUTH", "-sep", " ", "-element", "LastName,Initials")
-
-		// positional indices for title, abstract, and other text fields - TITL, ABST, TEXT
-
-		acc = append(acc, "-block", "PMCExtract", "-article", "TITLE/TEXT")
-		acc = append(acc, "-block", "PMCExtract", "-abstract", "ABSTRACT/TEXT")
-		acc = append(acc, "-block", "PMCExtract", "-paragraph", "TEXT")
+		acc = append(acc, line)
 	}
 
 	return acc
 }
 
-// UPDATE CACHED INVERTED FILES IN INDICES DIRECTORY FROM LOCAL ARCHIVE FOLDERS
+// UPDATE CACHED INDEXED AND INVERTED-INDEX FILES FROM LOCAL ARCHIVE FOLDERS
 
-// remove cached inverted index components with:
-//
-//   find /Volumes/cachet/Indices -name \*.inv.gz -type f -delete
-
-// examineFolder collects two-digit subdirectories, xml files, and e2x files
-func examineFolder(base, path string) ([]string, []string, []string) {
+// examineFolder collects two-digit subdirectories, xml files, e2x files, and inv files
+func examineFolder(base, path string) ([]string, []string, []string, []string) {
 
 	dir := filepath.Join(base, path)
 
 	contents, err := os.ReadDir(dir)
 	if err != nil {
-		return nil, nil, nil
+		return nil, nil, nil, nil
 	}
 
 	isTwoDigits := func(str string) bool {
@@ -312,6 +221,7 @@ func examineFolder(base, path string) ([]string, []string, []string) {
 	var dirs []string
 	var xmls []string
 	var e2xs []string
+	var invs []string
 
 	for _, item := range contents {
 		name := item.Name()
@@ -326,10 +236,12 @@ func examineFolder(base, path string) ([]string, []string, []string) {
 			xmls = append(xmls, name)
 		} else if strings.HasSuffix(name, ".e2x.gz") {
 			e2xs = append(e2xs, name)
+		} else if strings.HasSuffix(name, ".inv.gz") {
+			invs = append(invs, name)
 		}
 	}
 
-	return dirs, xmls, e2xs
+	return dirs, xmls, e2xs, invs
 }
 
 // gzFileToString reads selected gzipped file, uncompressing and saving contents as string
@@ -397,7 +309,7 @@ func stringToGzFile(base, path, file, str string) {
 	// for small files, use regular gzip
 	zpr, err := gzip.NewWriterLevel(fl, gzip.BestSpeed)
 	if err != nil {
-		fmt.Fprintf(os.Stderr, "\nERROR: Unable to create compressor\n")
+		DisplayError("Unable to create compressor")
 		os.Exit(1)
 	}
 
@@ -435,70 +347,47 @@ type e2IndexConsumer func(inp <-chan XMLRecord) <-chan XMLRecord
 
 // IncrementalIndex creates or updates missing cached .e2x.gz indexed files,
 // e.g., /Index/02/53/025393.e2x.gz for /Archive/02/53/93/*.xml.gz
-func IncrementalIndex(archiveBase, indexBase, db, pfx string, csmr e2IndexConsumer) <-chan string {
+func IncrementalIndex(db, pfx, ptrn string, dotMax int, csmr e2IndexConsumer) <-chan string {
 
 	if csmr == nil {
 		return nil
 	}
 
-	if archiveBase == "" {
-
-		// if not passed as an argument, obtain archive base path from environment variable
-		base := os.Getenv("EDIRECT_PUBMED_MASTER")
-		if base != "" {
-			if !strings.HasSuffix(base, "/") {
-				base += "/"
-			}
-
-			archiveBase = base + "Archive"
-		}
-
-		if archiveBase == "" {
-			fmt.Fprintf(os.Stderr, "\nERROR: EDIRECT_PUBMED_MASTER environment variable is not set\n\n")
-			os.Exit(1)
-		}
+	if db == "" {
+		db = "pubmed"
 	}
 
-	if indexBase == "" {
+	// obtain paths from environment variable(s)
+	archiveBase := GetLocalArchiveFolder(db, "Archive", true)
+	indexBase := GetLocalArchiveFolder(db, "Index", true)
 
-		// if not passed as an argument, obtain index base path from environment variable
-		base := os.Getenv("EDIRECT_PUBMED_WORKING")
-		if base != "" {
-			if !strings.HasSuffix(base, "/") {
-				base += "/"
-			}
-
-			indexBase = base + "Index"
-		}
-
-		if indexBase == "" {
-			fmt.Fprintf(os.Stderr, "\nERROR: EDIRECT_PUBMED_WORKING environment variable is not set\n\n")
-			os.Exit(1)
-		}
+	if archiveBase == "" || indexBase == "" {
+		DisplayError("Unable to get local archive or index paths")
+		os.Exit(1)
 	}
 
 	// check to make sure local archive is mounted
 	_, err := os.Stat(archiveBase)
 	if err != nil && os.IsNotExist(err) {
-		fmt.Fprintf(os.Stderr, "\nERROR: Local archive and search index is not mounted\n\n")
+		DisplayError("Local archive and search index is not mounted")
 		os.Exit(1)
 	}
 
 	// check to make sure local index is mounted
 	_, err = os.Stat(indexBase)
 	if err != nil && os.IsNotExist(err) {
-		fmt.Fprintf(os.Stderr, "\nERROR: Local incremental index is not mounted\n\n")
+		DisplayError("Local incremental index is not mounted")
 		os.Exit(1)
 	}
 
 	// visitArchiveFolders sends an Archive leaf folder path plus the file base names
-	// contained in it down a channel, e.g., "02/53/93/", "2539300", "2539301", ..., for
+	// contained in it down a channel, e.g., [ "02/53/93", "2539300", "2539301", ..., ] for
 	// Archive/02/53/93/*.xml.gz
 	visitArchiveFolders := func(archiveBase string) <-chan []string {
 
-		out := make(chan []string, ChanDepth())
+		out := make(chan []string, chanDepth)
 		if out == nil {
-			fmt.Fprintf(os.Stderr, "\nERROR: Unable to create archive explorer channel\n")
+			DisplayError("Unable to create archive explorer channel")
 			os.Exit(1)
 		}
 
@@ -508,7 +397,7 @@ func IncrementalIndex(archiveBase, indexBase, db, pfx string, csmr e2IndexConsum
 		// visitSubFolders recurses to leaf directories
 		visitSubFolders = func(base, path string, out chan<- []string) {
 
-			dirs, xmls, _ := examineFolder(base, path)
+			dirs, xmls, _, _ := examineFolder(base, path)
 
 			// recursively explore subdirectories
 			if dirs != nil {
@@ -535,22 +424,7 @@ func IncrementalIndex(archiveBase, indexBase, db, pfx string, csmr e2IndexConsum
 
 			if len(xmls) > 1 {
 				// sort fields in alphabetical or numeric order
-				sort.Slice(xmls, func(i, j int) bool {
-					// numeric sort on strings checks lengths first
-					if IsAllDigits(xmls[i]) && IsAllDigits(xmls[j]) {
-						lni := len(xmls[i])
-						lnj := len(xmls[j])
-						// shorter string is numerically less, assuming no leading zeros
-						if lni < lnj {
-							return true
-						}
-						if lni > lnj {
-							return false
-						}
-					}
-					// same length or non-numeric, can now do string comparison on contents
-					return xmls[i] < xmls[j]
-				})
+				slices.SortFunc(xmls, CompareAlphaOrNumericKeys)
 			}
 
 			var res []string
@@ -564,7 +438,7 @@ func IncrementalIndex(archiveBase, indexBase, db, pfx string, csmr e2IndexConsum
 
 			defer close(out)
 
-			dirs, _, _ := examineFolder(base, "")
+			dirs, _, _, _ := examineFolder(base, "")
 
 			// iterate through top directories
 			for _, top := range dirs {
@@ -572,6 +446,11 @@ func IncrementalIndex(archiveBase, indexBase, db, pfx string, csmr e2IndexConsum
 				if IsAllDigits(top) && len(top) == 2 {
 					visitSubFolders(base, top, out)
 				}
+
+				// force garbage collection
+				runtime.GC()
+
+				runtime.Gosched()
 			}
 		}
 
@@ -585,9 +464,9 @@ func IncrementalIndex(archiveBase, indexBase, db, pfx string, csmr e2IndexConsum
 	// only passing those files that need to be (re)indexed
 	filterIndexFolders := func(indexBase string, inp <-chan []string) <-chan XMLRecord {
 
-		out := make(chan XMLRecord, ChanDepth())
+		out := make(chan XMLRecord, chanDepth)
 		if out == nil {
-			fmt.Fprintf(os.Stderr, "\nERROR: Unable to create index filter channel\n")
+			DisplayError("Unable to create index filter channel")
 			os.Exit(1)
 		}
 
@@ -621,7 +500,7 @@ func IncrementalIndex(archiveBase, indexBase, db, pfx string, csmr e2IndexConsum
 				}
 
 				for _, pmid := range pmids {
-					// increment index so unshuffler can restore order of resuls
+					// increment index so unshuffler can restore order of results
 					idx++
 
 					// send PMID (unindexed file base name) down channel
@@ -638,9 +517,9 @@ func IncrementalIndex(archiveBase, indexBase, db, pfx string, csmr e2IndexConsum
 
 	cleanIndexFiles := func(inp <-chan XMLRecord) <-chan XMLRecord {
 
-		out := make(chan XMLRecord, ChanDepth())
+		out := make(chan XMLRecord, chanDepth)
 		if out == nil {
-			fmt.Fprintf(os.Stderr, "\nERROR: Unable to create index cleaner channel\n")
+			DisplayError("Unable to create index cleaner channel")
 			os.Exit(1)
 		}
 
@@ -658,7 +537,7 @@ func IncrementalIndex(archiveBase, indexBase, db, pfx string, csmr e2IndexConsum
 					continue
 				}
 
-				// clean up white space between stop tag and next start tag
+				// clean up white space between stop tag and next start tag, replacing with a single newline
 				str = re.ReplaceAllString(str, ">\n<")
 
 				if !strings.HasSuffix(str, "\n") {
@@ -672,7 +551,7 @@ func IncrementalIndex(archiveBase, indexBase, db, pfx string, csmr e2IndexConsum
 		var wg sync.WaitGroup
 
 		// launch multiple index cleaner goroutines
-		for i := 0; i < NumServe(); i++ {
+		for range numProcs {
 			wg.Add(1)
 			go indexCleaner(&wg, indexBase, inp, out)
 		}
@@ -688,9 +567,9 @@ func IncrementalIndex(archiveBase, indexBase, db, pfx string, csmr e2IndexConsum
 
 	combineIndexFiles := func(indexBase string, dotMax int, inp <-chan XMLRecord) <-chan string {
 
-		out := make(chan string, ChanDepth())
+		out := make(chan string, chanDepth)
 		if out == nil {
-			fmt.Fprintf(os.Stderr, "\nERROR: Unable to create index combiner channel\n")
+			DisplayError("Unable to create index combiner channel")
 			os.Exit(1)
 		}
 
@@ -730,8 +609,8 @@ func IncrementalIndex(archiveBase, indexBase, db, pfx string, csmr e2IndexConsum
 
 			verbose := false
 			// set verbose flag from environment variable
-			env := os.Getenv("EDIRECT_PUBMED_VERBOSE")
-			if env == "Y" || env == "y" {
+			env := os.Getenv("EDIRECT_LOCAL_VERBOSE")
+			if env == "Y" || env == "y" || env == "true" {
 				verbose = true
 			}
 
@@ -791,15 +670,19 @@ func IncrementalIndex(archiveBase, indexBase, db, pfx string, csmr e2IndexConsum
 		return out
 	}
 
-	// show a dot every 200 .e2x files, generated from up to 20000 .xml files
-	dotMax := 200
-	if db == "pmc" {
-		dotMax = 50
+	if dotMax == 0 {
+		// show a dot every 200 .e2x files, generated from up to 20000 .xml files
+		dotMax = 200
+		if db == "pmc" {
+			dotMax = 50
+		} else if db == "taxonomy" {
+			dotMax = 500
+		}
 	}
 
 	vrfq := visitArchiveFolders(archiveBase)
 	vifq := filterIndexFolders(indexBase, vrfq)
-	strq := CreateFetchers(archiveBase, "", pfx, ".xml", true, vifq)
+	strq := CreateFetchers(archiveBase, db, pfx, ".xml", ptrn, true, vifq)
 	// callback passes cmds and transform values as closures to xtract createConsumers
 	tblq := csmr(strq)
 	// clean up XML (no measured benefit to adding next record size prefix)
@@ -828,9 +711,9 @@ func InvertIndexedFile(inp <-chan string) <-chan string {
 			return nil
 		}
 
-		out := make(chan []string, ChanDepth())
+		out := make(chan []string, chanDepth)
 		if out == nil {
-			fmt.Fprintf(os.Stderr, "\nERROR: Unable to create dispenser channel\n")
+			DisplayError("Unable to create dispenser channel")
 			os.Exit(1)
 		}
 
@@ -939,7 +822,7 @@ func InvertIndexedFile(inp <-chan string) <-chan string {
 		var wg sync.WaitGroup
 
 		// launch multiple dispenser goroutines
-		for i := 0; i < NumServe(); i++ {
+		for range numProcs {
 			wg.Add(1)
 			go xmlDispenser(&wg, inp, out)
 		}
@@ -970,9 +853,9 @@ func InvertIndexedFile(inp <-chan string) <-chan string {
 			return nil
 		}
 
-		out := make(chan XMLRecord, ChanDepth())
+		out := make(chan XMLRecord, chanDepth)
 		if out == nil {
-			fmt.Fprintf(os.Stderr, "\nERROR: Unable to create inverter channel\n")
+			DisplayError("Unable to create inverter channel")
 			os.Exit(1)
 		}
 
@@ -1011,38 +894,13 @@ func InvertIndexedFile(inp <-chan string) <-chan string {
 				buffer.WriteString("<InvIDs>\n")
 
 				// sort fields in alphabetical order
-				var keys []string
-				for ky := range fields {
-					keys = append(keys, ky)
-				}
-				sort.Slice(keys, func(i, j int) bool { return keys[i] < keys[j] })
+				keys := slices.Sorted(maps.Keys(fields))
 
 				for _, fld := range keys {
 
 					positions := fields[fld]
 
-					var arry []string
-
-					for item := range positions {
-						arry = append(arry, item)
-					}
-
-					if len(arry) > 1 {
-						sort.Slice(arry, func(i, j int) bool {
-							// numeric sort on strings checks lengths first
-							lni := len(arry[i])
-							lnj := len(arry[j])
-							// shorter string is numerically less, assuming no leading zeros
-							if lni < lnj {
-								return true
-							}
-							if lni > lnj {
-								return false
-							}
-							// same length, can now do string comparison on contents
-							return arry[i] < arry[j]
-						})
-					}
+					arry := slices.SortedFunc(maps.Keys(positions), CompareAlphaOrNumericKeys)
 
 					// print list of UIDs, skipping duplicates
 					prev := ""
@@ -1092,7 +950,7 @@ func InvertIndexedFile(inp <-chan string) <-chan string {
 		var wg sync.WaitGroup
 
 		// launch multiple inverter goroutines
-		for i := 0; i < NumServe(); i++ {
+		for range numProcs {
 			wg.Add(1)
 			go xmlInverter(&wg, inp, out)
 		}
@@ -1112,9 +970,9 @@ func InvertIndexedFile(inp <-chan string) <-chan string {
 			return nil
 		}
 
-		out := make(chan string, ChanDepth())
+		out := make(chan string, chanDepth)
 		if out == nil {
-			fmt.Fprintf(os.Stderr, "\nERROR: Unable to create resolver channel\n")
+			DisplayError("Unable to create resolver channel")
 			os.Exit(1)
 		}
 
@@ -1133,15 +991,7 @@ func InvertIndexedFile(inp <-chan string) <-chan string {
 				inverted[curr.Ident] = curr.Text
 			}
 
-			var ordered []string
-
-			for item := range inverted {
-				ordered = append(ordered, item)
-			}
-
-			if len(ordered) > 1 {
-				sort.Slice(ordered, func(i, j int) bool { return ordered[i] < ordered[j] })
-			}
+			ordered := slices.Sorted(maps.Keys(inverted))
 
 			// iterate through alphabetized results
 			for _, curr := range ordered {
@@ -1173,44 +1023,32 @@ func InvertIndexedFile(inp <-chan string) <-chan string {
 }
 
 // IncrementalInvert creates or updates missing cached .inv.gz inverted index files
-func IncrementalInvert(indexBase, invertBase, db string) <-chan string {
+func IncrementalInvert(db string, dotMax int) <-chan string {
+
+	if db == "" {
+		db = "pubmed"
+	}
+
+	// obtain paths from environment variable(s)
+	indexBase := GetLocalArchiveFolder(db, "Index", true)
+	invertBase := GetLocalArchiveFolder(db, "Invert", true)
 
 	if indexBase == "" || invertBase == "" {
-
-		// if not passed as an argument, obtain index base path from environment variable
-		base := os.Getenv("EDIRECT_PUBMED_WORKING")
-		if base != "" {
-			if !strings.HasSuffix(base, "/") {
-				base += "/"
-			}
-
-			indexBase = base + "Index"
-			invertBase = base + "Invert"
-		}
-
-		if indexBase == "" {
-			fmt.Fprintf(os.Stderr, "\nERROR: EDIRECT_PUBMED_WORKING environment variable is not set\n\n")
-			os.Exit(1)
-		}
+		DisplayError("Unable to get local index or invert paths")
+		os.Exit(1)
 	}
 
 	// check to make sure local index directory is mounted
 	_, err := os.Stat(indexBase)
 	if err != nil && os.IsNotExist(err) {
-		fmt.Fprintf(os.Stderr, "\nERROR: Local index directory is not mounted\n\n")
+		DisplayError("Local incremental index is not mounted")
 		os.Exit(1)
 	}
 
 	// check to make sure local invert directory is mounted
 	_, err = os.Stat(invertBase)
 	if err != nil && os.IsNotExist(err) {
-		fmt.Fprintf(os.Stderr, "\nERROR: Local invert directory is not mounted\n\n")
-		os.Exit(1)
-	}
-
-	out := make(chan string, ChanDepth())
-	if out == nil {
-		fmt.Fprintf(os.Stderr, "\nERROR: Unable to create inverter channel\n")
+		DisplayError("Local invert directory is not mounted")
 		os.Exit(1)
 	}
 
@@ -1220,24 +1058,21 @@ func IncrementalInvert(indexBase, invertBase, db string) <-chan string {
 			return nil
 		}
 
-		out := make(chan string, ChanDepth())
+		out := make(chan string, chanDepth)
 		if out == nil {
-			fmt.Fprintf(os.Stderr, "\nERROR: Unable to create index fetcher channel\n")
+			DisplayError("Unable to create index fetcher channel")
 			os.Exit(1)
 		}
 
 		// e2xFetcher reads indexed XML from file
 		e2xFetcher := func(wg *sync.WaitGroup, inp <-chan string, out chan<- string) {
 
-			// report when more records to process
 			defer wg.Done()
 
 			for file := range inp {
 
 				txt := gzFileToString(file)
 
-				runtime.Gosched()
-
 				out <- txt
 			}
 		}
@@ -1245,7 +1080,7 @@ func IncrementalInvert(indexBase, invertBase, db string) <-chan string {
 		var wg sync.WaitGroup
 
 		// launch multiple fetcher goroutines
-		for i := 0; i < NumServe(); i++ {
+		for range numProcs {
 			wg.Add(1)
 			go e2xFetcher(&wg, inp, out)
 		}
@@ -1259,129 +1094,222 @@ func IncrementalInvert(indexBase, invertBase, db string) <-chan string {
 		return out
 	}
 
-	// invert one set of 250 indexed files, each representing 1000 pubmed or pmc records
-	invertIndexFiles := func(target string, filenames []string) {
+	invertIndexFiles := func(invBase string, inp <-chan []string) <-chan string {
 
-		s2cq := SliceToChan(filenames)
-		idfq := indexFetchers(s2cq)
-		// indexDispenser | indexInverter | indexResolver
-		iifq := InvertIndexedFile(idfq)
-
-		var buffer strings.Builder
-
-		for str := range iifq {
-			buffer.WriteString(str)
+		out := make(chan string, chanDepth)
+		if out == nil {
+			DisplayError("Unable to create index inverter channel")
+			os.Exit(1)
 		}
 
-		txt := buffer.String()
-		if txt == "" {
-			fmt.Fprintf(os.Stderr, "Empty %s\n", target)
-			return
+		invertIndexSubset := func(wg *sync.WaitGroup, invBase string, inp <-chan []string, out chan<- string) {
+
+			defer wg.Done()
+
+			for data := range inp {
+
+				// path is first element in slice
+				path := data[0]
+				// "02/53"
+
+				// followed by e2x file paths
+				filenames := data[1:]
+
+				invPath := path[:3]
+				// "02/"
+
+				invFile := strings.Replace(path, "/", "", -1)
+				// "0253"
+
+				s2cq := SliceToChan(filenames)
+				idfq := indexFetchers(s2cq)
+				// indexDispenser | indexInverter | indexResolver
+				iifq := InvertIndexedFile(idfq)
+
+				var buffer strings.Builder
+
+				for str := range iifq {
+					buffer.WriteString(str)
+				}
+
+				txt := buffer.String()
+				if txt == "" {
+					fmt.Fprintf(os.Stderr, "Empty %s\n", invPath+invFile)
+					return
+				}
+
+				// save to target file
+				stringToGzFile(invertBase, invPath, invFile+".inv.gz", txt)
+
+				out <- invFile + ".inv.gz"
+			}
 		}
 
-		// save to target file
-		stringToGzFile(invertBase, "", target, txt)
+		var wg sync.WaitGroup
+
+		// launch inverter goroutines (use 3 to numProcs: with numServe, PMC will eventually overflow memory)
+		for range numProcs {
+			wg.Add(1)
+			go invertIndexSubset(&wg, invBase, inp, out)
+		}
+
+		// launch separate anonymous goroutine to wait until all inverters are done
+		go func() {
+			wg.Wait()
+			close(out)
+		}()
+
+		return out
 	}
 
-	visitIndexSubset := func(indexBase, invertBase string, out chan<- string) {
+	// visitIndexFolders sends an Index leaf folder name, e.g., "02/53",
+	// plus full paths to all component index files, down a channel, but
+	// only if there is no existing Invert file, e.g., Invert/02/0253.inv.gz
+	visitIndexFolders := func(indexBase, invertBase string) <-chan []string {
 
-		defer close(out)
-
-		num := 0
-
-		fileBase := "pubmed"
-		if db == "pmc" {
-			fileBase = "pmc"
+		out := make(chan []string, chanDepth)
+		if out == nil {
+			DisplayError("Unable to create index explorer channel")
+			os.Exit(1)
 		}
 
-		visitSubFolders := func(path, fr, to string) {
+		// mutex to protect access to rollingCount and rollingColumn variables
+		var vlock sync.Mutex
 
-			// increment output file number
-			num++
+		if dotMax == 0 {
+			// show a dot every 10 .inv files that were deleted for regeneration,
+			// representing from up to 1000 .e2x files, some also just regenerated
+			dotMax = 10
+		}
 
-			sfx := fmt.Sprintf("%03d", num)
-			fname := fileBase + sfx + ".inv.gz"
+		rollingCount := 0
+		rollingColumn := 0
 
-			target := filepath.Join(invertBase, fname)
+		countSuccess := func() {
+
+			vlock.Lock()
+
+			rollingCount++
+			if rollingCount >= dotMax {
+				rollingCount = 0
+				// print dot (progress monitor)
+				fmt.Fprintf(os.Stderr, ".")
+				rollingColumn++
+				if rollingColumn > 49 {
+					// print newline after 50 dots
+					fmt.Fprintf(os.Stderr, "\n")
+					rollingColumn = 0
+				}
+			}
+
+			vlock.Unlock()
+		}
+
+		verbose := false
+		// set verbose flag from environment variable
+		env := os.Getenv("EDIRECT_LOCAL_VERBOSE")
+		if env == "Y" || env == "y" || env == "true" {
+			verbose = true
+		}
+
+		// recursive definition
+		var visitSubFolders func(idxBase, invBase, path string, out chan<- []string)
+
+		// visitSubFolders recurses to leaf directories
+		visitSubFolders = func(idxBase, invBase, path string, out chan<- []string) {
+
+			dirs, _, e2xs, _ := examineFolder(idxBase, path)
+
+			// recursively explore subdirectories
+			if dirs != nil {
+				for _, dr := range dirs {
+					sub := filepath.Join(path, dr)
+					visitSubFolders(idxBase, invBase, sub, out)
+				}
+				return
+			}
+
+			// looking for leaf Index directory with at least one *.e2x.gz file
+			if e2xs == nil {
+				return
+			}
+
+			// "02/53"
+
+			invPath := path[:2]
+			// "02"
+
+			invFile := strings.Replace(path, "/", "", -1)
+			// "0253"
+
+			target := filepath.Join(invBase, invPath, invFile+".inv.gz")
 
-			// incremental inverted index file is removed when records in relevant range are archived or deleted
 			_, err := os.Stat(target)
 			if err == nil {
-				// if inverted index file exists, no need to recreate
+				// if inverted index file exists for the indexed folder, no need to recreate
 				return
 			}
 
-			mids, _, _ := examineFolder(indexBase, path)
+			var res []string
 
-			var filenames []string
+			// first string is indexed folder path, to become inverted file name
+			res = append(res, path)
 
-			for _, fld := range mids {
+			for _, file := range e2xs {
 
-				// filter second-level directories by range
-				if fld < fr {
-					continue
-				}
-				if fld > to {
-					break
-				}
+				// path for indexed file within current leaf folder
+				fpath := filepath.Join(idxBase, path, file)
 
-				sub := filepath.Join(path, fld)
-				_, _, e2xs := examineFolder(indexBase, sub)
-
-				for _, file := range e2xs {
-
-					// path for indexed file within current leaf folder
-					fpath := filepath.Join(indexBase, sub, file)
-
-					filenames = append(filenames, fpath)
-				}
+				res = append(res, fpath)
 			}
 
-			if len(filenames) < 1 {
-				return
+			if verbose {
+				fmt.Fprintf(os.Stderr, "INV %s\n", invFile)
+			} else {
+				// progress monitor
+				countSuccess()
 			}
 
-			invertIndexFiles(fname, filenames)
-
-			out <- fname
+			out <- res
 		}
 
-		// get top-level directories
-		dirs, _, _ := examineFolder(indexBase, "")
+		visitIndexSubset := func(idxBase, invBase string, out chan<- []string) {
 
-		// iterate through top directories
-		for _, dr := range dirs {
+			defer close(out)
 
-			// skip Sentinels folder
-			if !IsAllDigits(dr) {
-				continue
+			dirs, _, _, _ := examineFolder(idxBase, "")
+
+			// iterate through top directories
+			for _, top := range dirs {
+				if IsAllDigits(top) && len(top) == 2 {
+					visitSubFolders(idxBase, invBase, top, out)
+				}
+
+				// force garbage collection
+				runtime.GC()
+
+				runtime.Gosched()
 			}
 
-			// collect inverted index files from groups of pubmed or pmc .xml files
-			if db == "" || db == "pubmed" {
-				// 250000 pubmed records
-				visitSubFolders(dr, "00", "24")
-				visitSubFolders(dr, "25", "49")
-				visitSubFolders(dr, "50", "74")
-				visitSubFolders(dr, "75", "99")
-			} else if db == "pmc" {
-				// 10000 pmc records
-				visitSubFolders(dr, "00", "09")
-				visitSubFolders(dr, "10", "19")
-				visitSubFolders(dr, "20", "29")
-				visitSubFolders(dr, "30", "39")
-				visitSubFolders(dr, "40", "49")
-				visitSubFolders(dr, "50", "59")
-				visitSubFolders(dr, "60", "69")
-				visitSubFolders(dr, "70", "79")
-				visitSubFolders(dr, "80", "89")
-				visitSubFolders(dr, "90", "99")
+			if rollingColumn > 0 {
+				vlock.Lock()
+				fmt.Fprintf(os.Stderr, "\n")
+				vlock.Unlock()
 			}
 		}
+
+		// launch single index visitor goroutine
+		go visitIndexSubset(indexBase, invertBase, out)
+
+		return out
 	}
 
-	// launch single index visitor goroutine
-	go visitIndexSubset(indexBase, invertBase, out)
+	vafq := visitIndexFolders(indexBase, invertBase)
+	out := invertIndexFiles(invertBase, vafq)
+	if vafq == nil || out == nil {
+		DisplayError("Unable to create visitIndexFolders channel")
+		os.Exit(1)
+	}
 
 	return out
 }
