@@ -36,22 +36,23 @@ import (
 	"fmt"
 	"github.com/klauspost/pgzip"
 	"io"
+	"maps"
 	"os"
 	"path/filepath"
 	"runtime"
 	"runtime/debug"
-	"sort"
+	"slices"
 	"strings"
 	"sync"
 )
 
 // Plex allows distribution of indexing
 type Plex struct {
-	Which int
+	Sibs  []string
 	Ident string
 	Text  string
+	Which int
 	Index int
-	Sibs  []string
 }
 
 // PlexHeap methods satisfy heap.Interface
@@ -90,13 +91,13 @@ func CreatePresenters(files []string) []<-chan Plex {
 
 	numFiles := len(files)
 	if numFiles < 1 {
-		fmt.Fprintf(os.Stderr, "\nERROR: Not enough inverted files to merge\n")
+		DisplayError("Not enough inverted files to merge")
 		os.Exit(1)
 	}
 
 	chns := make([]<-chan Plex, numFiles)
 	if chns == nil {
-		fmt.Fprintf(os.Stderr, "\nERROR: Unable to create presenter channel array\n")
+		DisplayError("Unable to create presenter channel array")
 		os.Exit(1)
 	}
 
@@ -108,7 +109,7 @@ func CreatePresenters(files []string) []<-chan Plex {
 
 		f, err := os.Open(fileName)
 		if err != nil {
-			fmt.Fprintf(os.Stderr, "\nERROR: Unable to open input file '%s'\n", fileName)
+			DisplayError("Unable to open input file '%s'", fileName)
 			os.Exit(1)
 		}
 
@@ -128,13 +129,13 @@ func CreatePresenters(files []string) []<-chan Plex {
 		if iszip {
 			brd := bufio.NewReader(f)
 			if brd == nil {
-				fmt.Fprintf(os.Stderr, "\nERROR: Unable to create buffered reader on '%s'\n", fileName)
+				DisplayError("Unable to create buffered reader on '%s'", fileName)
 				os.Exit(1)
 			}
 			// using parallel pgzip for better performance on large files
 			zpr, err := pgzip.NewReader(brd)
 			if err != nil {
-				fmt.Fprintf(os.Stderr, "\nERROR: Unable to create decompressor on '%s'\n", fileName)
+				DisplayError("Unable to create decompressor on '%s'", fileName)
 				os.Exit(1)
 			}
 
@@ -145,10 +146,10 @@ func CreatePresenters(files []string) []<-chan Plex {
 			in = zpr
 		}
 
-		rdr := CreateXMLStreamer(in)
+		rdr := CreateXMLStreamer(in, nil)
 
 		if rdr == nil {
-			fmt.Fprintf(os.Stderr, "\nERROR: Unable to create XML Block Reader\n")
+			DisplayError("Unable to create XML Block Reader")
 			os.Exit(1)
 		}
 
@@ -159,16 +160,16 @@ func CreatePresenters(files []string) []<-chan Plex {
 			func(str string) {
 				id := FindIdentifier(str[:], "InvDocument", find)
 
-				out <- Plex{fileNum, id, str, 0, nil}
+				out <- Plex{Which: fileNum, Ident: id, Text: str, Index: 0, Sibs: nil}
 			})
 	}
 
 	// launch multiple presenter goroutines
 	for i, str := range files {
 
-		chn := make(chan Plex, ChanDepth())
+		chn := make(chan Plex, chanDepth)
 		if chn == nil {
-			fmt.Fprintf(os.Stderr, "\nERROR: Unable to create presenter channel\n")
+			DisplayError("Unable to create presenter channel")
 			os.Exit(1)
 		}
 
@@ -189,9 +190,9 @@ func CreateManifold(inp []<-chan Plex) <-chan Plex {
 		return nil
 	}
 
-	out := make(chan Plex, ChanDepth())
+	out := make(chan Plex, chanDepth)
 	if out == nil {
-		fmt.Fprintf(os.Stderr, "\nERROR: Unable to create manifold channel\n")
+		DisplayError("Unable to create manifold channel")
 		os.Exit(1)
 	}
 
@@ -237,7 +238,7 @@ func CreateManifold(inp []<-chan Plex) <-chan Plex {
 
 					rec++
 					// send set from previous identifier to output channel
-					out <- Plex{0, prevIdent, "", rec, arry}
+					out <- Plex{Which: 0, Ident: prevIdent, Text: "", Index: rec, Sibs: arry}
 
 					// empty the slice
 					arry = nil
@@ -264,7 +265,7 @@ func CreateManifold(inp []<-chan Plex) <-chan Plex {
 
 			rec++
 			// send last record
-			out <- Plex{0, prevIdent, "", rec, arry}
+			out <- Plex{Which: 0, Ident: prevIdent, Text: "", Index: rec, Sibs: arry}
 
 			arry = nil
 		}
@@ -283,9 +284,9 @@ func CreateFusers(inp <-chan XMLRecord) <-chan Plex {
 		return nil
 	}
 
-	out := make(chan Plex, ChanDepth())
+	out := make(chan Plex, chanDepth)
 	if out == nil {
-		fmt.Fprintf(os.Stderr, "\nERROR: Unable to create fuser channel\n")
+		DisplayError("Unable to create fuser channel")
 		os.Exit(1)
 	}
 
@@ -330,7 +331,7 @@ func CreateFusers(inp <-chan XMLRecord) <-chan Plex {
 	var wg sync.WaitGroup
 
 	// launch multiple fuser goroutines
-	for i := 0; i < NumServe(); i++ {
+	for range numProcs {
 		wg.Add(1)
 		go xmlFuser(&wg, inp, out)
 	}
@@ -340,11 +341,7 @@ func CreateFusers(inp <-chan XMLRecord) <-chan Plex {
 		wg.Wait()
 
 		// sort id keys in alphabetical order
-		var keys []string
-		for ky := range inverted {
-			keys = append(keys, ky)
-		}
-		sort.Slice(keys, func(i, j int) bool { return keys[i] < keys[j] })
+		keys := slices.Sorted(maps.Keys(inverted))
 
 		rec := 0
 
@@ -354,7 +351,7 @@ func CreateFusers(inp <-chan XMLRecord) <-chan Plex {
 
 			rec++
 			// send array of records with same identifier to output channel
-			out <- Plex{0, id, "", rec, arry}
+			out <- Plex{Which: 0, Ident: id, Text: "", Index: rec, Sibs: arry}
 
 			// empty the slice
 			arry = nil
@@ -366,6 +363,48 @@ func CreateFusers(inp <-chan XMLRecord) <-chan Plex {
 	return out
 }
 
+// CreateJoiners repackages adjacent indices for the same term without fusing
+func CreateJoiners(inp <-chan Plex) <-chan XMLRecord {
+
+	if inp == nil {
+		return nil
+	}
+
+	out := make(chan XMLRecord, chanDepth)
+	if out == nil {
+		DisplayError("Unable to create merger channel")
+		os.Exit(1)
+	}
+
+	// xmlJoiner unpacks adjacent InvDocument records with the same identifier
+	xmlJoiner := func(inp <-chan Plex, out chan<- XMLRecord) {
+
+		defer close(out)
+
+		for plx := range inp {
+
+			rec := plx.Index
+			key := plx.Ident
+			data := plx.Sibs
+
+			if len(data) < 1 {
+				continue
+			}
+
+			for _, str := range data {
+				out <- XMLRecord{Index: rec, Ident: key, Text: str[:]}
+			}
+
+			runtime.Gosched()
+		}
+	}
+
+	// launch single joiner goroutine
+	go xmlJoiner(inp, out)
+
+	return out
+}
+
 // CreateMergers combines collected indices for the same term
 func CreateMergers(inp <-chan Plex) <-chan XMLRecord {
 
@@ -373,9 +412,9 @@ func CreateMergers(inp <-chan Plex) <-chan XMLRecord {
 		return nil
 	}
 
-	out := make(chan XMLRecord, ChanDepth())
+	out := make(chan XMLRecord, chanDepth)
 	if out == nil {
-		fmt.Fprintf(os.Stderr, "\nERROR: Unable to create merger channel\n")
+		DisplayError("Unable to create merger channel")
 		os.Exit(1)
 	}
 
@@ -413,11 +452,7 @@ func CreateMergers(inp <-chan Plex) <-chan XMLRecord {
 			}
 
 			// sort fields in alphabetical order
-			var keys []string
-			for ky := range fields {
-				keys = append(keys, ky)
-			}
-			sort.Slice(keys, func(i, j int) bool { return keys[i] < keys[j] })
+			keys := slices.Sorted(maps.Keys(fields))
 
 			var res []string
 
@@ -425,28 +460,7 @@ func CreateMergers(inp <-chan Plex) <-chan XMLRecord {
 
 				positions := fields[fld]
 
-				var arry []string
-
-				for item := range positions {
-					arry = append(arry, item)
-				}
-
-				if len(arry) > 1 {
-					sort.Slice(arry, func(i, j int) bool {
-						// numeric sort on strings checks lengths first
-						lni := len(arry[i])
-						lnj := len(arry[j])
-						// shorter string is numerically less, assuming no leading zeros
-						if lni < lnj {
-							return true
-						}
-						if lni > lnj {
-							return false
-						}
-						// same length, can now do string comparison on contents
-						return arry[i] < arry[j]
-					})
-				}
+				arry := slices.SortedFunc(maps.Keys(positions), CompareAlphaOrNumericKeys)
 
 				// print list of UIDs, skipping duplicates
 				last := ""
@@ -495,7 +509,7 @@ func CreateMergers(inp <-chan Plex) <-chan XMLRecord {
 	var wg sync.WaitGroup
 
 	// launch multiple merger goroutines
-	for i := 0; i < NumServe(); i++ {
+	for range numProcs {
 		wg.Add(1)
 		go xmlMerger(&wg, inp, out)
 	}
@@ -510,19 +524,34 @@ func CreateMergers(inp <-chan Plex) <-chan XMLRecord {
 }
 
 // CreateSplitter distributes adjacent records with the same identifier prefix
-func CreateSplitter(mergePath string, zipp, isLink bool, inp <-chan XMLRecord) <-chan string {
+func CreateSplitter(mergePath, db string, zipp, isLink bool, inp <-chan XMLRecord) <-chan string {
 
 	if inp == nil {
 		return nil
 	}
 
-	out := make(chan string, ChanDepth())
-	if out == nil {
-		fmt.Fprintf(os.Stderr, "\nERROR: Unable to create splitter channel\n")
+	// obtain paths from environment variable(s)
+	mergeBase := GetLocalArchiveFolder(db, "Merged", true)
+
+	if mergeBase == "" {
+		DisplayError("Unable to get local merge path")
 		os.Exit(1)
 	}
 
-	openSaver := func(mergePath, key string, zipp bool) (*os.File, *bufio.Writer, *pgzip.Writer) {
+	// check to make sure local merge directory is mounted
+	_, err := os.Stat(mergeBase)
+	if err != nil && os.IsNotExist(err) {
+		DisplayError("Local merge directory '%s' is not mounted", mergeBase)
+		os.Exit(1)
+	}
+
+	out := make(chan string, chanDepth)
+	if out == nil {
+		DisplayError("Unable to create splitter channel")
+		os.Exit(1)
+	}
+
+	openSaver := func(mergeBase, key string, zipp bool) (*os.File, *bufio.Writer, *pgzip.Writer) {
 
 		var (
 			fl   *os.File
@@ -536,7 +565,7 @@ func CreateSplitter(mergePath string, zipp, isLink bool, inp <-chan XMLRecord) <
 			sfx += ".gz"
 		}
 
-		fpath := filepath.Join(mergePath, key+sfx)
+		fpath := filepath.Join(mergeBase, key+sfx)
 		if fpath == "" {
 			return nil, nil, nil
 		}
@@ -567,7 +596,7 @@ func CreateSplitter(mergePath string, zipp, isLink bool, inp <-chan XMLRecord) <
 		// create buffered writer layer
 		wrtr = bufio.NewWriter(out)
 		if wrtr == nil {
-			fmt.Fprintf(os.Stderr, "Unable to create bufio.NewWriter\n")
+			DisplayError("Unable to create bufio.NewWriter")
 			return nil, nil, nil
 		}
 
@@ -658,7 +687,7 @@ func CreateSplitter(mergePath string, zipp, isLink bool, inp <-chan XMLRecord) <
 
 			if fl == nil {
 				// open initial file
-				fl, wrtr, zpr = openSaver(mergePath, currTag, zipp)
+				fl, wrtr, zpr = openSaver(mergeBase, currTag, zipp)
 				if wrtr == nil {
 					continue
 				}
@@ -687,7 +716,7 @@ func CreateSplitter(mergePath string, zipp, isLink bool, inp <-chan XMLRecord) <
 				runtime.Gosched()
 
 				// open next file
-				fl, wrtr, zpr = openSaver(mergePath, currTag, zipp)
+				fl, wrtr, zpr = openSaver(mergeBase, currTag, zipp)
 				if wrtr == nil {
 					continue
 				}
