@@ -1356,6 +1356,37 @@ var ncbi4naToIupac = map[int]string{
 	255: "NN",
 }
 
+// CleanDoctype accepts full doctype objects or just the inner contents
+func CleanDoctype(doctype string) string {
+
+	doctype = strings.TrimSpace(doctype)
+	doctype = strings.TrimPrefix(doctype, "<")
+	doctype = strings.TrimPrefix(doctype, "!")
+	doctype = strings.TrimPrefix(doctype, "DOCTYPE")
+	doctype = strings.TrimSuffix(doctype, ">")
+	doctype = strings.TrimSpace(doctype)
+
+	doctype = "<!DOCTYPE " + doctype + ">"
+
+	return doctype
+}
+
+// CleanXmltype accepts full xml objects or just the inner contents
+func CleanXmltype(xmltype string) string {
+
+	xmltype = strings.TrimSpace(xmltype)
+	xmltype = strings.TrimPrefix(xmltype, "<")
+	xmltype = strings.TrimPrefix(xmltype, "?")
+	xmltype = strings.TrimPrefix(xmltype, "xml")
+	xmltype = strings.TrimSuffix(xmltype, ">")
+	xmltype = strings.TrimSuffix(xmltype, "?")
+	xmltype = strings.TrimSpace(xmltype)
+
+	xmltype = "<?xml " + xmltype + " ?>"
+
+	return xmltype
+}
+
 // use RelaxString to convert non-alphanumeric characters to spaces
 
 // CleanupAuthor fixes misused letters and accents
@@ -1488,7 +1519,7 @@ func CleanupContents(str string, ascii, amper, mixed bool) string {
 	}
 	if allowEmbed {
 		if amper {
-			str = RepairEncodedMarkup(str)
+			str = CleanupEncodedMarkup(str)
 		}
 	}
 	if doScript {
@@ -1513,7 +1544,7 @@ func CleanupContents(str string, ascii, amper, mixed bool) string {
 				str = RepairScriptMarkup(str, CONCISE)
 				str = RepairTableMarkup(str, SPACE)
 				// call RepairScriptMarkup before RemoveEmbeddedMarkup
-				str = RemoveEmbeddedMarkup(str)
+				str = RemoveEmbeddedMarkup(str, NOMARKUP)
 			}
 		}
 		if ascii && HasBadSpace(str) {
@@ -1574,6 +1605,162 @@ func CleanupContents(str string, ascii, amper, mixed bool) string {
 	return str
 }
 
+// CleanupEncodedMarkup removes ampersand-encoded markup
+func CleanupEncodedMarkup(str string) string {
+
+	// convert &lt;sup&gt; to <sup> (html subset)
+	// convert &amp;#181; to &#181; (but not further - use html.UnescapeString)
+	// convert &amp;amp;amp;amp;amp;amp;amp;lt; to &lt;
+	// remove </sub><sub> or </sup><sup> (internals)
+
+	var buffer strings.Builder
+
+	lookAhead := func(txt string, to int) string {
+		mx := len(txt)
+		if to > mx {
+			to = mx
+		}
+		pos := strings.Index(txt[:to], "gt;")
+		if pos > 0 {
+			to = pos + 3
+		}
+		return txt[:to]
+	}
+
+	skip := 0
+
+	for i, ch := range str {
+		if skip > 0 {
+			skip--
+			continue
+		}
+		if ch == '<' {
+			// remove internal tags in runs of subscripts or superscripts
+			if strings.HasPrefix(str[i:], "</sub><sub>") || strings.HasPrefix(str[i:], "</sup><sup>") {
+				skip = 10
+				continue
+			}
+			buffer.WriteRune(ch)
+			continue
+		} else if ch != '&' {
+			buffer.WriteRune(ch)
+			continue
+		} else if strings.HasPrefix(str[i:], "&lt;") {
+			sub := lookAhead(str[i:], 14)
+			txt, ok := htmlRepair[sub]
+			if ok {
+				adv := len(sub) - 1
+				// do not convert if flanked by spaces - it may be a scientific symbol,
+				// e.g., fragments <i> in PMID 9698410, or escaped <b> and <d> tags used
+				// to indicate stem position in letters in PMID 21892341
+				if i < 1 || str[i-1] != ' ' || !strings.HasPrefix(str[i+adv:], "; ") {
+					buffer.WriteString(txt)
+					skip = adv
+					continue
+				}
+			}
+		} else if strings.HasPrefix(str[i:], "&amp;") {
+			if strings.HasPrefix(str[i:], "&amp;lt;") {
+				sub := lookAhead(str[i:], 22)
+				txt, ok := htmlRepair[sub]
+				if ok {
+					buffer.WriteString(txt)
+					skip = len(sub) - 1
+					continue
+				} else {
+					buffer.WriteString("&lt;")
+					skip = 7
+					continue
+				}
+			} else if strings.HasPrefix(str[i:], "&amp;gt;") {
+				buffer.WriteString("&gt;")
+				skip = 7
+				continue
+			} else {
+				skip = 4
+				j := i + 5
+				// remove runs of multiply-encoded ampersands
+				for strings.HasPrefix(str[j:], "amp;") {
+					skip += 4
+					j += 4
+				}
+				// then look for special symbols used in PubMed records
+				if strings.HasPrefix(str[j:], "lt;") {
+					buffer.WriteString("&lt;")
+					skip += 3
+				} else if strings.HasPrefix(str[j:], "gt;") {
+					buffer.WriteString("&gt;")
+					skip += 3
+				} else if strings.HasPrefix(str[j:], "frac") {
+					buffer.WriteString("&frac")
+					skip += 4
+				} else if strings.HasPrefix(str[j:], "plusmn") {
+					buffer.WriteString("&plusmn")
+					skip += 6
+				} else if strings.HasPrefix(str[j:], "acute") {
+					buffer.WriteString("&acute")
+					skip += 5
+				} else if strings.HasPrefix(str[j:], "aacute") {
+					buffer.WriteString("&aacute")
+					skip += 6
+				} else if strings.HasPrefix(str[j:], "rsquo") {
+					buffer.WriteString("&rsquo")
+					skip += 5
+				} else if strings.HasPrefix(str[j:], "lsquo") {
+					buffer.WriteString("&lsquo")
+					skip += 5
+				} else if strings.HasPrefix(str[j:], "micro") {
+					buffer.WriteString("&micro")
+					skip += 5
+				} else if strings.HasPrefix(str[j:], "oslash") {
+					buffer.WriteString("&oslash")
+					skip += 6
+				} else if strings.HasPrefix(str[j:], "kgr") {
+					buffer.WriteString("&kgr")
+					skip += 3
+				} else if strings.HasPrefix(str[j:], "apos") {
+					buffer.WriteString("&apos")
+					skip += 4
+				} else if strings.HasPrefix(str[j:], "quot") {
+					buffer.WriteString("&quot")
+					skip += 4
+				} else if strings.HasPrefix(str[j:], "alpha") {
+					buffer.WriteString("&alpha")
+					skip += 5
+				} else if strings.HasPrefix(str[j:], "beta") {
+					buffer.WriteString("&beta")
+					skip += 4
+				} else if strings.HasPrefix(str[j:], "gamma") {
+					buffer.WriteString("&gamma")
+					skip += 5
+				} else if strings.HasPrefix(str[j:], "Delta") {
+					buffer.WriteString("&Delta")
+					skip += 5
+				} else if strings.HasPrefix(str[j:], "phi") {
+					buffer.WriteString("&phi")
+					skip += 3
+				} else if strings.HasPrefix(str[j:], "ge") {
+					buffer.WriteString("&ge")
+					skip += 2
+				} else if strings.HasPrefix(str[j:], "sup2") {
+					buffer.WriteString("&sup2")
+					skip += 4
+				} else if strings.HasPrefix(str[j:], "#") {
+					buffer.WriteString("&")
+				} else {
+					buffer.WriteString("&amp;")
+				}
+				continue
+			}
+		}
+
+		// if loop not continued by any preceding test, print character
+		buffer.WriteRune(ch)
+	}
+
+	return buffer.String()
+}
+
 // CleanupPlain removes embedded mixed-content markup tags
 func CleanupPlain(str string, wrp bool) string {
 
@@ -1583,7 +1770,7 @@ func CleanupPlain(str string, wrp bool) string {
 
 	str = strings.Replace(str, "\n", " ", -1)
 
-	str = RemoveEmbeddedMarkup(str)
+	str = RemoveEmbeddedMarkup(str, NOMARKUP)
 	str = TransformAccents(str, false, false)
 
 	if HasUnicodeMarkup(str) {
@@ -1627,7 +1814,7 @@ func CleanupProse(str string, wrp bool) string {
 		str = html.UnescapeString(str)
 	}
 
-	str = RemoveEmbeddedMarkup(str)
+	str = RemoveEmbeddedMarkup(str, SPACE)
 	str = TransformAccents(str, false, false)
 	str = FixMisusedLetters(str, true, false, true)
 	str = TransformAccents(str, false, false)
@@ -1684,11 +1871,11 @@ func CleanupQuery(str string, exactMatch, removeBrackets bool) string {
 
 	if removeBrackets {
 		if HasAngleOrAmpersandEncoding(str) {
-			str = RepairEncodedMarkup(str)
+			str = CleanupEncodedMarkup(str)
 			str = RepairScriptMarkup(str, SPACE)
 			str = RepairMathMLMarkup(str, SPACE)
 			// RemoveEmbeddedMarkup must be called before UnescapeString, which was suppressed in ExploreElements
-			str = RemoveEmbeddedMarkup(str)
+			str = RemoveEmbeddedMarkup(str, NOMARKUP)
 		}
 	}
 
@@ -1977,7 +2164,7 @@ func FlattenMathML(str string, policy int) string {
 
 	str = strings.TrimSpace(str)
 
-	// str = RemoveEmbeddedMarkup(str)
+	// str = RemoveEmbeddedMarkup(str, NOMARKUP)
 
 	return str
 }
@@ -2766,12 +2953,12 @@ func PrepareForIndexing(str string, doHomoglyphs, isAuthor, isProse, spellGreek,
 		str = CleanupBadSpaces(str)
 	}
 	if HasAngleOrAmpersandEncoding(str) {
-		str = RepairEncodedMarkup(str)
+		str = CleanupEncodedMarkup(str)
 		str = RepairTableMarkup(str, SPACE)
 		str = RepairScriptMarkup(str, SPACE)
 		str = RepairMathMLMarkup(str, SPACE)
 		// RemoveEmbeddedMarkup must be called before UnescapeString, which was suppressed in ExploreElements
-		str = RemoveEmbeddedMarkup(str)
+		str = RemoveEmbeddedMarkup(str, SPACE)
 	}
 
 	if HasAmpOrNotASCII(str) {
@@ -2846,17 +3033,41 @@ func RelaxString(str string) string {
 }
 
 // RemoveEmbeddedMarkup removes internal mixed-content sections
-func RemoveEmbeddedMarkup(str string) string {
+func RemoveEmbeddedMarkup(str string, policy int) string {
 
 	inContent := true
-	var buffer strings.Builder
+	shouldPad := false
+	var (
+		curr   rune
+		prev   rune
+		buffer strings.Builder
+	)
 
 	for _, ch := range str {
 		if ch == '<' {
 			inContent = false
 		} else if ch == '>' {
+			if !inContent {
+				shouldPad = true
+			}
 			inContent = true
 		} else if inContent {
+			prev = curr
+			curr = ch
+			if shouldPad {
+				if unicode.IsLetter(prev) && unicode.IsLetter(curr) {
+					switch policy {
+					case SPACE:
+						buffer.WriteRune(' ')
+					}
+				} else if unicode.IsDigit(prev) && unicode.IsDigit(curr) {
+					switch policy {
+					case SPACE:
+						buffer.WriteRune(' ')
+					}
+				}
+			}
+			shouldPad = false
 			buffer.WriteRune(ch)
 		}
 	}
@@ -2925,162 +3136,6 @@ func RemoveHTMLDecorations(str string) string {
 	return str
 }
 
-// RepairEncodedMarkup removes ampersand-encoded markup
-func RepairEncodedMarkup(str string) string {
-
-	// convert &lt;sup&gt; to <sup> (html subset)
-	// convert &amp;#181; to &#181; (but not further - use html.UnescapeString)
-	// convert &amp;amp;amp;amp;amp;amp;amp;lt; to &lt;
-	// remove </sub><sub> or </sup><sup> (internals)
-
-	var buffer strings.Builder
-
-	lookAhead := func(txt string, to int) string {
-		mx := len(txt)
-		if to > mx {
-			to = mx
-		}
-		pos := strings.Index(txt[:to], "gt;")
-		if pos > 0 {
-			to = pos + 3
-		}
-		return txt[:to]
-	}
-
-	skip := 0
-
-	for i, ch := range str {
-		if skip > 0 {
-			skip--
-			continue
-		}
-		if ch == '<' {
-			// remove internal tags in runs of subscripts or superscripts
-			if strings.HasPrefix(str[i:], "</sub><sub>") || strings.HasPrefix(str[i:], "</sup><sup>") {
-				skip = 10
-				continue
-			}
-			buffer.WriteRune(ch)
-			continue
-		} else if ch != '&' {
-			buffer.WriteRune(ch)
-			continue
-		} else if strings.HasPrefix(str[i:], "&lt;") {
-			sub := lookAhead(str[i:], 14)
-			txt, ok := htmlRepair[sub]
-			if ok {
-				adv := len(sub) - 1
-				// do not convert if flanked by spaces - it may be a scientific symbol,
-				// e.g., fragments <i> in PMID 9698410, or escaped <b> and <d> tags used
-				// to indicate stem position in letters in PMID 21892341
-				if i < 1 || str[i-1] != ' ' || !strings.HasPrefix(str[i+adv:], "; ") {
-					buffer.WriteString(txt)
-					skip = adv
-					continue
-				}
-			}
-		} else if strings.HasPrefix(str[i:], "&amp;") {
-			if strings.HasPrefix(str[i:], "&amp;lt;") {
-				sub := lookAhead(str[i:], 22)
-				txt, ok := htmlRepair[sub]
-				if ok {
-					buffer.WriteString(txt)
-					skip = len(sub) - 1
-					continue
-				} else {
-					buffer.WriteString("&lt;")
-					skip = 7
-					continue
-				}
-			} else if strings.HasPrefix(str[i:], "&amp;gt;") {
-				buffer.WriteString("&gt;")
-				skip = 7
-				continue
-			} else {
-				skip = 4
-				j := i + 5
-				// remove runs of multiply-encoded ampersands
-				for strings.HasPrefix(str[j:], "amp;") {
-					skip += 4
-					j += 4
-				}
-				// then look for special symbols used in PubMed records
-				if strings.HasPrefix(str[j:], "lt;") {
-					buffer.WriteString("&lt;")
-					skip += 3
-				} else if strings.HasPrefix(str[j:], "gt;") {
-					buffer.WriteString("&gt;")
-					skip += 3
-				} else if strings.HasPrefix(str[j:], "frac") {
-					buffer.WriteString("&frac")
-					skip += 4
-				} else if strings.HasPrefix(str[j:], "plusmn") {
-					buffer.WriteString("&plusmn")
-					skip += 6
-				} else if strings.HasPrefix(str[j:], "acute") {
-					buffer.WriteString("&acute")
-					skip += 5
-				} else if strings.HasPrefix(str[j:], "aacute") {
-					buffer.WriteString("&aacute")
-					skip += 6
-				} else if strings.HasPrefix(str[j:], "rsquo") {
-					buffer.WriteString("&rsquo")
-					skip += 5
-				} else if strings.HasPrefix(str[j:], "lsquo") {
-					buffer.WriteString("&lsquo")
-					skip += 5
-				} else if strings.HasPrefix(str[j:], "micro") {
-					buffer.WriteString("&micro")
-					skip += 5
-				} else if strings.HasPrefix(str[j:], "oslash") {
-					buffer.WriteString("&oslash")
-					skip += 6
-				} else if strings.HasPrefix(str[j:], "kgr") {
-					buffer.WriteString("&kgr")
-					skip += 3
-				} else if strings.HasPrefix(str[j:], "apos") {
-					buffer.WriteString("&apos")
-					skip += 4
-				} else if strings.HasPrefix(str[j:], "quot") {
-					buffer.WriteString("&quot")
-					skip += 4
-				} else if strings.HasPrefix(str[j:], "alpha") {
-					buffer.WriteString("&alpha")
-					skip += 5
-				} else if strings.HasPrefix(str[j:], "beta") {
-					buffer.WriteString("&beta")
-					skip += 4
-				} else if strings.HasPrefix(str[j:], "gamma") {
-					buffer.WriteString("&gamma")
-					skip += 5
-				} else if strings.HasPrefix(str[j:], "Delta") {
-					buffer.WriteString("&Delta")
-					skip += 5
-				} else if strings.HasPrefix(str[j:], "phi") {
-					buffer.WriteString("&phi")
-					skip += 3
-				} else if strings.HasPrefix(str[j:], "ge") {
-					buffer.WriteString("&ge")
-					skip += 2
-				} else if strings.HasPrefix(str[j:], "sup2") {
-					buffer.WriteString("&sup2")
-					skip += 4
-				} else if strings.HasPrefix(str[j:], "#") {
-					buffer.WriteString("&")
-				} else {
-					buffer.WriteString("&amp;")
-				}
-				continue
-			}
-		}
-
-		// if loop not continued by any preceding test, print character
-		buffer.WriteRune(ch)
-	}
-
-	return buffer.String()
-}
-
 // RepairMathMLMarkup removes MathML embedded markup symbols
 func RepairMathMLMarkup(str string, policy int) string {
 
