@@ -31,7 +31,6 @@
 package eutils
 
 import (
-	"fmt"
 	"html"
 	"os"
 	"strings"
@@ -199,7 +198,7 @@ func NormalizeXML(rdr <-chan XMLBlock, db string) <-chan string {
 
 	out := make(chan string, chanDepth)
 	if out == nil {
-		fmt.Fprintf(os.Stderr, "Unable to create normalize channel\n")
+		DisplayError("Unable to create normalize channel")
 		os.Exit(1)
 	}
 
@@ -211,7 +210,7 @@ func NormalizeXML(rdr <-chan XMLBlock, db string) <-chan string {
 		tknq := CreateTokenizer(rdr)
 
 		if tknq == nil {
-			fmt.Fprintf(os.Stderr, "\nERROR: Unable to create normalize tokenizer\n")
+			DisplayError("Unable to create normalize tokenizer")
 			os.Exit(1)
 		}
 
@@ -343,9 +342,7 @@ func NormalizeXML(rdr <-chan XMLBlock, db string) <-chan string {
 			case CONTENTTAG:
 				if isDocsum {
 					if db == "pubmed" && prevName == "Title" {
-						if strings.Contains(name, "&") ||
-							strings.Contains(name, "<") ||
-							strings.Contains(name, ">") {
+						if strings.ContainsAny(name, "<>&") {
 							ctype := tkn.Cont
 							name = CleanupContents(name, (ctype&ASCII) != 0, (ctype&AMPER) != 0, (ctype&MIXED) != 0)
 							name = cleanPubmedEncoding(name)
@@ -361,7 +358,7 @@ func NormalizeXML(rdr <-chan XMLBlock, db string) <-chan string {
 							}
 							name = html.UnescapeString(name)
 							// reencode < and > to avoid breaking XML
-							if strings.Contains(name, "<") || strings.Contains(name, ">") || strings.Contains(name, "&") {
+							if strings.ContainsAny(name, "<>&") {
 								name = rfix.Replace(name)
 							}
 						}
@@ -401,7 +398,7 @@ func NormalizeXML(rdr <-chan XMLBlock, db string) <-chan string {
 						}
 						name = html.UnescapeString(name)
 						// reencode < and > to avoid breaking XML
-						if strings.Contains(name, "<") || strings.Contains(name, ">") || strings.Contains(name, "&") {
+						if strings.ContainsAny(name, "<>&") {
 							name = rfix.Replace(name)
 						}
 					}
