@@ -40,7 +40,7 @@ import (
 	"io"
 	"os"
 	"path/filepath"
-	"sort"
+	"slices"
 	"strconv"
 	"strings"
 	"sync"
@@ -48,6 +48,49 @@ import (
 
 // POSTINGS FILE CREATION FROM MERGED INVERTED INDEX FILES
 
+// UintsToVarints converts a set of integers to compact variable byte length unsigned integers
+func UintsToVarints(vals []uint32) []byte {
+
+	var buffr []byte
+
+	appendOneVarint := func(val uint32) {
+
+		var buf []byte
+
+		buf = binary.AppendUvarint(buf, uint64(val))
+
+		// append variable byte length unsigned integer
+		buffr = append(buffr, buf...)
+	}
+
+	for _, val := range vals {
+		appendOneVarint(val)
+	}
+
+	return buffr
+}
+
+// VarintsToUints converts a set of variable byte length unsigned integers back to integers
+func VarintsToUints(buf []byte) []uint32 {
+
+	var data []uint32
+
+	buffr := buf[:]
+
+	for len(buffr) > 0 {
+		val, n := binary.Uvarint(buffr)
+		if n < 1 {
+			break
+		}
+
+		data = append(data, uint32(val))
+
+		buffr = buffr[n:]
+	}
+
+	return data
+}
+
 // CreatePromoters creates term lists and postings files from merged inverted
 // index files. All specified fields (e.g., "TITL TIAB YEAR TREE") are completed
 // in a single scan of the inverted files.
@@ -62,27 +105,38 @@ import (
 //
 // For position data, the .uqi file is parallel to the .pst file (one entry
 // for each PMID associated with a given term), and contains 32-bit offsets
-// to 16-bit paragraph position values.
+// to variable byte encoded paragraph position values.
 //
 // An extra entry at the end, pointing just past the end of data, allows
 // the length of a term, or the size of a postings list, or the number of
 // positions for a term in a given PMID, to be calculated as the difference
 // between two adjacent pointers.
 //
-// The number of term positions per PMID is the term frequency (TF). The
+// The number of term positions per PMID is the term frequency (TF). (This
+// now cannot be calculated without unpacking the variable length data.) The
 // number of PMIDs per term is the document frequency (DF). All that remains
 // for calculating TF-IDF term weights, which can support ranked retrieval,
 // is the total number of live PubMed documents, which could easily be saved
 // during indexing.
-func CreatePromoters(prom, fields string, isLink bool, files []string) <-chan string {
+func CreatePromoters(prom, db, fields string, isLink bool, files []string) <-chan string {
 
 	if files == nil {
 		return nil
 	}
 
-	out := make(chan string, ChanDepth())
+	// obtain paths from environment variable(s)
+	postingsBase := GetLocalArchiveFolder(db, "Postings", true)
+
+	// check to make sure local postings directory is mounted
+	_, err := os.Stat(postingsBase)
+	if err != nil && os.IsNotExist(err) {
+		DisplayError("Local postings directory '%s' is not mounted", postingsBase)
+		os.Exit(1)
+	}
+
+	out := make(chan string, chanDepth)
 	if out == nil {
-		fmt.Fprintf(os.Stderr, "\nERROR: Unable to create promoter channel\n")
+		DisplayError("Unable to create promoter channel")
 		os.Exit(1)
 	}
 
@@ -95,7 +149,7 @@ func CreatePromoters(prom, fields string, isLink bool, files []string) <-chan st
 
 		f, err := os.Open(fileName)
 		if err != nil {
-			fmt.Fprintf(os.Stderr, "\nERROR: Unable to open input file '%s'\n", fileName)
+			DisplayError("Unable to open input file '%s'", fileName)
 			os.Exit(1)
 		}
 
@@ -115,13 +169,13 @@ func CreatePromoters(prom, fields string, isLink bool, files []string) <-chan st
 		if iszip {
 			brd := bufio.NewReader(f)
 			if brd == nil {
-				fmt.Fprintf(os.Stderr, "\nERROR: Unable to create buffered reader on '%s'\n", fileName)
+				DisplayError("Unable to create buffered reader on '%s'", fileName)
 				os.Exit(1)
 			}
 			// using parallel pgzip for better performance on large files
 			zpr, err := pgzip.NewReader(brd)
 			if err != nil {
-				fmt.Fprintf(os.Stderr, "\nERROR: Unable to create decompressor on '%s'\n", fileName)
+				DisplayError("Unable to create decompressor on '%s'", fileName)
 				os.Exit(1)
 			}
 
@@ -132,10 +186,10 @@ func CreatePromoters(prom, fields string, isLink bool, files []string) <-chan st
 			in = zpr
 		}
 
-		rdr := CreateXMLStreamer(in)
+		rdr := CreateXMLStreamer(in, nil)
 
 		if rdr == nil {
-			fmt.Fprintf(os.Stderr, "\nERROR: Unable to create XML Block Reader\n")
+			DisplayError("Unable to create XML Block Reader")
 			os.Exit(1)
 		}
 
@@ -159,7 +213,7 @@ func CreatePromoters(prom, fields string, isLink bool, files []string) <-chan st
 
 					// convert UID string to integer
 					if content == "" {
-						fmt.Fprintf(os.Stderr, "\nERROR: Empty UID for term '%s'\n", term)
+						DisplayError("Empty UID for term '%s'", term)
 						return
 					}
 					value, err := strconv.ParseInt(content, 10, 32)
@@ -205,6 +259,40 @@ func CreatePromoters(prom, fields string, isLink bool, files []string) <-chan st
 
 		retlength := len("\n")
 
+		addPositions := func(attr string) {
+
+			binary.Write(&uqidList, binary.LittleEndian, ofstPos)
+
+			var vals []uint32
+
+			atrs := strings.Split(attr, ",")
+			for _, att := range atrs {
+				if att == "" {
+					continue
+				}
+				value, err := strconv.ParseInt(att, 10, 32)
+				if err != nil {
+					fmt.Fprintf(os.Stderr, "%s\n", err.Error())
+					return
+				}
+				vals = append(vals, uint32(value))
+			}
+
+			if len(vals) < 1 {
+				return
+			}
+
+			buffr := UintsToVarints(vals)
+			if len(buffr) < 1 {
+				return
+			}
+
+			// positions are variable byte length encoded unsigned integers
+			binary.Write(&ofstList, binary.LittleEndian, buffr)
+
+			ofstPos += int32(len(buffr))
+		}
+
 		addOnePosting := func(term string, data []int32, atts []string) {
 
 			tlength := len(term)
@@ -236,24 +324,8 @@ func CreatePromoters(prom, fields string, isLink bool, files []string) <-chan st
 
 			// write term offset list for each UID
 			for _, attr := range atts {
-
-				binary.Write(&uqidList, binary.LittleEndian, ofstPos)
-
-				atrs := strings.Split(attr, ",")
-				atln := len(atrs)
-				for _, att := range atrs {
-					if att == "" {
-						continue
-					}
-					value, err := strconv.ParseInt(att, 10, 32)
-					if err != nil {
-						fmt.Fprintf(os.Stderr, "%s\n", err.Error())
-						return
-					}
-					binary.Write(&ofstList, binary.LittleEndian, int16(value))
-				}
-
-				ofstPos += int32(atln * 2)
+				// now using variable byte length unsigned integers instead of uint16
+				addPositions(attr)
 			}
 		}
 
@@ -297,7 +369,7 @@ func CreatePromoters(prom, fields string, isLink bool, files []string) <-chan st
 
 		writeFiveFiles := func(field, key string) {
 
-			dpath, ky := PostingPath(prom, field, key, isLink)
+			dpath, ky := PostingPath(postingsBase, field, key, isLink)
 			if dpath == "" {
 				return
 			}
@@ -315,8 +387,8 @@ func CreatePromoters(prom, fields string, isLink bool, files []string) <-chan st
 
 			writeFile(dpath, ky+"."+field+".mst", indxList)
 
-			// do not write position index and offset data files
-			// for fields with no position attributes recorded
+			// only write position index and offset data files
+			// for fields with position attributes recorded
 			if uqidList.Len() > 0 && ofstList.Len() > 0 {
 
 				writeFile(dpath, ky+"."+field+".uqi", uqidList)
@@ -457,7 +529,7 @@ type Master struct {
 // Arrays contains postings lists and word offsets
 type Arrays struct {
 	Data []int32
-	Ofst [][]int16
+	Ofst [][]uint32
 	Dist int
 }
 
@@ -592,7 +664,7 @@ func readPositionIndex(dpath, key, field string, offset int32, size int32) []int
 	return data
 }
 
-func readOffsetData(dpath, key, field string, offset int32, size int32) []int16 {
+func readOffsetData(dpath, key, field string, offset int32, size int32) []byte {
 
 	inFile, _ := commonOpenFile(dpath, key+"."+field+".ofs")
 	if inFile == nil {
@@ -601,8 +673,8 @@ func readOffsetData(dpath, key, field string, offset int32, size int32) []int16
 
 	defer inFile.Close()
 
-	data := make([]int16, size/2)
-	if data == nil || len(data) < 1 {
+	data := make([]byte, size)
+	if data == nil {
 		return nil
 	}
 
@@ -623,9 +695,9 @@ func readOffsetData(dpath, key, field string, offset int32, size int32) []int16
 
 func readMasterIndexFuture(dpath, key, field string) <-chan []Master {
 
-	out := make(chan []Master, ChanDepth())
+	out := make(chan []Master, chanDepth)
 	if out == nil {
-		fmt.Fprintf(os.Stderr, "\nERROR: Unable to create master index channel\n")
+		DisplayError("Unable to create master index channel")
 		os.Exit(1)
 	}
 
@@ -647,9 +719,9 @@ func readMasterIndexFuture(dpath, key, field string) <-chan []Master {
 
 func readTermListFuture(dpath, key, field string) <-chan []byte {
 
-	out := make(chan []byte, ChanDepth())
+	out := make(chan []byte, chanDepth)
 	if out == nil {
-		fmt.Fprintf(os.Stderr, "\nERROR: Unable to create term list channel\n")
+		DisplayError("Unable to create term list channel")
 		os.Exit(1)
 	}
 
@@ -669,7 +741,7 @@ func readTermListFuture(dpath, key, field string) <-chan []byte {
 	return out
 }
 
-func getPostingIDs(prom, term, field string, simple, isLink bool) ([]int32, [][]int16) {
+func getPostingIDs(prom, term, field string, simple, isLink bool) ([]int32, [][]uint32) {
 
 	dpath, key := PostingPath(prom, field, term, isLink)
 	if dpath == "" {
@@ -779,7 +851,7 @@ func getPostingIDs(prom, term, field string, simple, isLink bool) ([]int32, [][]
 					i++
 				}
 
-				sort.Slice(fused, func(i, j int) bool { return fused[i] < fused[j] })
+				slices.Sort(fused)
 
 				return fused, nil
 			}
@@ -803,13 +875,13 @@ func getPostingIDs(prom, term, field string, simple, isLink bool) ([]int32, [][]
 				return nil, nil
 			}
 
-			combo := make(map[int32][]int16)
+			combo := make(map[int32][]uint32)
 
-			addPositions := func(uid int32, pos int16) {
+			addPositions := func(uid int32, pos uint32) {
 
 				arrs, ok := combo[uid]
 				if !ok {
-					arrs = make([]int16, 0, 1)
+					arrs = make([]uint32, 0, 1)
 				}
 				arrs = append(arrs, pos)
 				combo[uid] = arrs
@@ -818,10 +890,11 @@ func getPostingIDs(prom, term, field string, simple, isLink bool) ([]int32, [][]
 			// populate array of positions per UID
 			for i, j, k := 0, 1, int32(0); i < ulen-1; i++ {
 				uid := data[i]
-				num := (uqis[j] - uqis[i]) / 2
+				num := (uqis[j] - uqis[i])
 				j++
-				for q := k; q < k+num; q++ {
-					addPositions(uid, ofst[q])
+				posns := VarintsToUints(ofst[k : k+num])
+				for _, pos := range posns {
+					addPositions(uid, pos)
 				}
 				k += num
 			}
@@ -835,10 +908,10 @@ func getPostingIDs(prom, term, field string, simple, isLink bool) ([]int32, [][]
 				i++
 			}
 
-			sort.Slice(fused, func(i, j int) bool { return fused[i] < fused[j] })
+			slices.Sort(fused)
 
-			// make array of int16 arrays, populate for each UID
-			arrs := make([][]int16, ulen-1)
+			// make array of uint32 arrays, populate for each UID
+			arrs := make([][]uint32, ulen-1)
 			if arrs == nil {
 				return nil, nil
 			}
@@ -847,7 +920,7 @@ func getPostingIDs(prom, term, field string, simple, isLink bool) ([]int32, [][]
 				posn := combo[uid]
 
 				if len(posn) > 1 {
-					sort.Slice(posn, func(i, j int) bool { return posn[i] < posn[j] })
+					slices.Sort(posn)
 				}
 
 				arrs[j] = posn
@@ -894,17 +967,17 @@ func getPostingIDs(prom, term, field string, simple, isLink bool) ([]int32, [][]
 			return nil, nil
 		}
 
-		// make array of int16 arrays, populate for each UID
-		arrs := make([][]int16, ulen)
+		// make array of uint32 arrays, populate for each UID
+		arrs := make([][]uint32, ulen)
 		if arrs == nil || len(arrs) < 1 {
 			return nil, nil
 		}
 
 		// populate array of positions per UID
 		for i, j, k := 0, 1, int32(0); i < ulen-1; i++ {
-			num := (uqis[j] - uqis[i]) / 2
+			num := (uqis[j] - uqis[i])
 			j++
-			arrs[i] = ofst[k : k+num]
+			arrs[i] = VarintsToUints(ofst[k : k+num])
 			k += num
 		}
 
@@ -916,9 +989,9 @@ func getPostingIDs(prom, term, field string, simple, isLink bool) ([]int32, [][]
 
 func postingIDsFuture(base, term, field string, dist int, isLink bool) <-chan Arrays {
 
-	out := make(chan Arrays, ChanDepth())
+	out := make(chan Arrays, chanDepth)
 	if out == nil {
-		fmt.Fprintf(os.Stderr, "\nERROR: Unable to create postings channel\n")
+		DisplayError("Unable to create postings channel")
 		os.Exit(1)
 	}
 
@@ -940,7 +1013,7 @@ func postingIDsFuture(base, term, field string, dist int, isLink bool) <-chan Ar
 
 // BOOLEAN OPERATIONS FOR POSTINGS LISTS
 
-func extendPositionalIDs(N []int32, np [][]int16, M []int32, mp [][]int16, delta int, proc func(pn, pm []int16, dlt int16) []int16) ([]int32, [][]int16) {
+func extendPositionalIDs(N []int32, np [][]uint32, M []int32, mp [][]uint32, delta int, proc func(pn, pm []uint32, dlt uint32) []uint32) ([]int32, [][]uint32) {
 
 	if proc == nil {
 		return nil, nil
@@ -967,7 +1040,7 @@ func extendPositionalIDs(N []int32, np [][]int16, M []int32, mp [][]int16, delta
 	}
 
 	res := make([]int32, sz)
-	ofs := make([][]int16, sz)
+	ofs := make([][]uint32, sz)
 
 	if res == nil || len(res) < 1 || ofs == nil || len(ofs) < 1 {
 		return nil, nil
@@ -994,7 +1067,7 @@ func extendPositionalIDs(N []int32, np [][]int16, M []int32, mp [][]int16, delta
 			em = M[j]
 		} else {
 			// specific callbacks test position arrays to match terms by adjacency or phrases by proximity
-			adj := proc(np[i], mp[j], int16(delta))
+			adj := proc(np[i], mp[j], uint32(delta))
 			if adj != nil && len(adj) > 0 {
 				res[k] = en
 				ofs[k] = adj
