@@ -31,7 +31,6 @@
 package eutils
 
 import (
-	"fmt"
 	"os"
 	"strings"
 	"unicode"
@@ -52,14 +51,14 @@ import (
 
 // BMHSearcher for Boyer-Moore-Horspool search
 type BMHSearcher struct {
+	skipTable [256]int
+	pattern   string
+	patlen    int
 	sensitive bool
 	whole     bool
 	relaxed   bool
 	compress  bool
 	circular  bool
-	skipTable [256]int
-	pattern   string
-	patlen    int
 }
 
 // StringSearcher primes Boyer-Moore-Horspool displacement table
@@ -99,7 +98,7 @@ func StringSearcher(pattern string, caseSensitive, wholeWord, relaxed, compress,
 	for i := range skip {
 		skip[i] = patlen
 	}
-	for i := 0; i < last; i++ {
+	for i := range last {
 		ch := pattern[i]
 		if caseSensitive {
 			skip[ch] = last - i
@@ -142,14 +141,13 @@ func (srch *BMHSearcher) Search(text string, offset int, proc func(string, strin
 		text = " " + text + " "
 	}
 
-	// single pattern, use Boyer-Moore-Horspool algorithm.
-	// By experiment, this was slightly (but reproducibly) faster than the Boyer-Moore-Sunday variant.
+	// single pattern, use Boyer-Moore-Horspool algorithm
 
 	// original length of text before any duplication to simulate circularity
 	cutoff := len(text)
 
 	if cutoff < srch.patlen {
-		// fmt.Fprintf(os.Stderr, "ERROR: Search text is shorter than pattern\n")
+		// DisplayError("Search text is shorter than pattern")
 		return
 	}
 
@@ -233,7 +231,7 @@ func (srch *BMHSearcher) Matches(text string) <-chan int {
 
 	out := make(chan int, chanDepth)
 	if out == nil {
-		fmt.Fprintf(os.Stderr, "\nERROR: Unable to create string search matcher\n")
+		DisplayError("Unable to create string search matcher")
 		os.Exit(1)
 	}
 
@@ -268,19 +266,19 @@ type matchesEntry struct {
 
 type stateEntry struct {
 	transit []transitEntry
-	failure int
 	matches []matchesEntry
+	failure int
 }
 
 // FSMSearcher for Finite State Machine search
 type FSMSearcher struct {
+	stateArray []stateEntry
+	maxpatlen  int
 	sensitive  bool
 	whole      bool
 	relaxed    bool
 	compress   bool
 	circular   bool
-	stateArray []stateEntry
-	maxpatlen  int
 }
 
 const failState = -1
@@ -448,14 +446,14 @@ func fsmSearcher(patterns []string, caseSensitive, wholeWord, relaxed, compress,
 
 		alias := ""
 
-		if !caseSensitive {
-			txt = strings.ToLower(txt)
-		}
-
 		if isSequence {
 			txt, alias = SplitInTwoLeft(txt, ":")
 		}
 
+		if !caseSensitive {
+			txt = strings.ToLower(txt)
+		}
+
 		if relaxed {
 			txt = RelaxString(txt)
 		} else if compress {
@@ -648,7 +646,7 @@ func SequenceSearcher(patterns []string, isProtein, isCircular, topStrandOnly bo
 		expanded, overflowed := expandPattern(txt)
 
 		if overflowed {
-			fmt.Fprintf(os.Stderr, "ERROR: Ignoring pattern expansion of '%s' due to overflow\n", pat)
+			DisplayError("Ignoring pattern expansion of '%s' due to overflow", pat)
 			continue
 		}
 
@@ -670,7 +668,7 @@ func SequenceSearcher(patterns []string, isProtein, isCircular, topStrandOnly bo
 		}
 	}
 
-	return fsmSearcher(arry, false, false, false, true, true, isCircular, !topStrandOnly)
+	return fsmSearcher(arry, false, false, false, true, isCircular, true, !topStrandOnly)
 }
 
 // Search uses precomputed Searcher tables to search a string or sequence
@@ -698,7 +696,7 @@ func (srch *FSMSearcher) Search(text string, proc func(string, string, int) bool
 	cutoff := len(text)
 
 	if cutoff < srch.maxpatlen {
-		// fmt.Fprintf(os.Stderr, "ERROR: Search text is shorter than pattern\n")
+		// DisplayError("Search text is shorter than pattern")
 		return
 	}
 
