@@ -37,7 +37,7 @@ import (
 )
 
 // ValidateXML checks for well-formed XML
-func ValidateXML(rdr <-chan XMLBlock, fnd string, html bool) int {
+func ValidateXML(rdr <-chan XMLBlock, fnd string, html bool, max int) int {
 
 	if rdr == nil {
 		return 0
@@ -48,7 +48,7 @@ func ValidateXML(rdr <-chan XMLBlock, fnd string, html bool) int {
 	tknq := CreateTokenizer(rdr)
 
 	if tknq == nil {
-		fmt.Fprintf(os.Stderr, "\nERROR: Unable to create validator tokenizer\n")
+		DisplayError("Unable to create validator tokenizer")
 		os.Exit(1)
 	}
 
@@ -248,9 +248,13 @@ func ValidateXML(rdr <-chan XMLBlock, fnd string, html bool) int {
 
 	verifyLevel("", "", 0)
 
-	// raised maxDepth test from 30 because of PMC nesting depth of 97 levels
-	// in oa_comm_xml.PMC009xxxxxx.baseline.2022-12-18.tar.gz release file
-	if maxDepth > 100 {
+	// raised maxDepth test to 100 because of PMC nesting depth of 97 levels
+	// in oa_comm_xml.PMC009xxxxxx.baseline.2022-12-18.tar.gz release file,
+	// from complex math formulae markup in PMC9439944
+	if max <= 0 || max > 10000 {
+		max = 100
+	}
+	if maxDepth > max {
 		fmt.Fprintf(os.Stdout, "%s%8d\tMaximum nesting, %d levels\n", depthID, depthLine, maxDepth)
 	}
 
