@@ -34,13 +34,13 @@ import (
 	"bufio"
 	"bytes"
 	"container/heap"
-	"fmt"
 	"html"
 	"io"
+	"maps"
 	"os"
 	"runtime"
 	"runtime/debug"
-	"sort"
+	"slices"
 	"strconv"
 	"strings"
 	"sync"
@@ -57,7 +57,57 @@ type XMLBlock string
 // CreateXMLStreamer reads XML input into a channel of trimmed strings that are
 // then split by PartitionXML into individual records (which can be processed
 // concurrently), or parsed directly into a channel of tokens by CreateTokenizer.
-func CreateXMLStreamer(in io.Reader) <-chan XMLBlock {
+func CreateXMLStreamer(in io.Reader, inp <-chan string) <-chan XMLBlock {
+
+	if inp != nil {
+
+		out := make(chan XMLBlock, chanDepth)
+		if out == nil {
+			DisplayError("Unable to create XML block reader channel")
+			os.Exit(1)
+		}
+
+		bufferXMLChannel := func(inp <-chan string, out chan<- XMLBlock) {
+
+			// close channel when all records have been processed
+			defer close(out)
+
+			remainder := ""
+
+			for str := range inp {
+
+				idx := strings.LastIndex(str, ">")
+				// still need to check doStrict and call local version of htmlBehind
+				if idx >= 0 {
+					idx++
+					prev := remainder
+					remainder = str[idx:]
+					str = str[:idx]
+					rslt := prev + str
+					rslt = strings.TrimSpace(rslt)
+					rslt = CompressRunsOfSpaces(rslt)
+					rslt = strings.TrimPrefix(rslt, " ")
+					rslt = strings.TrimSuffix(rslt, " ")
+					out <- XMLBlock(rslt)
+				} else {
+					remainder += str
+				}
+			}
+
+			if remainder != "" {
+				remainder = strings.TrimSpace(remainder)
+				remainder = CompressRunsOfSpaces(remainder)
+				remainder = strings.TrimPrefix(remainder, " ")
+				remainder = strings.TrimSuffix(remainder, " ")
+				out <- XMLBlock(remainder)
+			}
+		}
+
+		// launch single buffering goroutine
+		go bufferXMLChannel(inp, out)
+
+		return out
+	}
 
 	if in == nil {
 		return nil
@@ -65,7 +115,7 @@ func CreateXMLStreamer(in io.Reader) <-chan XMLBlock {
 
 	out := make(chan XMLBlock, chanDepth)
 	if out == nil {
-		fmt.Fprintf(os.Stderr, "\nERROR: Unable to create XML block reader channel\n")
+		DisplayError("Unable to create XML block reader channel")
 		os.Exit(1)
 	}
 
@@ -131,7 +181,7 @@ func CreateXMLStreamer(in io.Reader) <-chan XMLBlock {
 			if err != nil {
 				if err != io.EOF {
 					// real error.
-					fmt.Fprintf(os.Stderr, "\n%sERROR: %s%s\n", RED, err.Error(), INIT)
+					DisplayError("%s", err.Error())
 					// ignore bytes - non-conforming implementations of io.Reader may
 					// return mangled data on non-EOF errors
 					isClosed = true
@@ -147,7 +197,7 @@ func CreateXMLStreamer(in io.Reader) <-chan XMLBlock {
 			}
 			if n < 0 {
 				// reality check - non-conforming implementations of io.Reader may return -1
-				fmt.Fprintf(os.Stderr, "\n%sERROR: io.Reader returned negative count %d%s\n", RED, n, INIT)
+				DisplayError("io.Reader returned negative count %d", n)
 				// treat as n == 0 in order to update file offset and avoid losing previous remainder
 				n = 0
 			}
@@ -281,7 +331,7 @@ func PartitionXML(pat, star string, turbo bool, inp <-chan XMLBlock, proc func(s
 	for i := range skip {
 		skip[i] = patlen
 	}
-	for i := 0; i < last; i++ {
+	for i := range last {
 		ch := pat[i]
 		skip[ch] = last - i
 	}
@@ -783,10 +833,10 @@ func PartitionXML(pat, star string, turbo bool, inp <-chan XMLBlock, proc func(s
 // that record. The Index field stores the record's original position in the
 // input stream. The Data field is used for binary compressed PubmedArticle XML.
 type XMLRecord struct {
-	Index int
+	Data  []byte
 	Ident string
 	Text  string
-	Data  []byte
+	Index int
 }
 
 // CreateXMLProducer partitions an XML set and sends records down a channel.
@@ -800,7 +850,7 @@ func CreateXMLProducer(pat, star string, turbo bool, rdr <-chan XMLBlock) <-chan
 
 	out := make(chan XMLRecord, chanDepth)
 	if out == nil {
-		fmt.Fprintf(os.Stderr, "\nERROR: Unable to create XML producer channel\n")
+		DisplayError("Unable to create XML producer channel")
 		os.Exit(1)
 	}
 
@@ -816,7 +866,7 @@ func CreateXMLProducer(pat, star string, turbo bool, rdr <-chan XMLBlock) <-chan
 		PartitionXML(pat, star, turbo, rdr,
 			func(str string) {
 				rec++
-				out <- XMLRecord{rec, "", str, nil}
+				out <- XMLRecord{Index: rec, Text: str}
 			})
 	}
 
@@ -862,7 +912,7 @@ func CreateXMLUnshuffler(inp <-chan XMLRecord) <-chan XMLRecord {
 
 	out := make(chan XMLRecord, chanDepth)
 	if out == nil {
-		fmt.Fprintf(os.Stderr, "\nERROR: Unable to create XML unshuffler channel\n")
+		DisplayError("Unable to create XML unshuffler channel")
 		os.Exit(1)
 	}
 
@@ -909,7 +959,7 @@ func CreateXMLUnshuffler(inp <-chan XMLRecord) <-chan XMLRecord {
 				}
 
 				// send even if empty to get all record counts for reordering
-				out <- XMLRecord{curr.Index, curr.Ident, curr.Text, curr.Data}
+				out <- XMLRecord{Index: curr.Index, Ident: curr.Ident, Text: curr.Text, Data: curr.Data}
 
 				// prevent ambiguous -limit filter from clogging heap (deprecated)
 				if curr.Index == next {
@@ -925,7 +975,7 @@ func CreateXMLUnshuffler(inp <-chan XMLRecord) <-chan XMLRecord {
 		for hp.Len() > 0 {
 			curr := heap.Pop(hp).(XMLRecord)
 
-			out <- XMLRecord{curr.Index, curr.Ident, curr.Text, curr.Data}
+			out <- XMLRecord{Index: curr.Index, Ident: curr.Ident, Text: curr.Text, Data: curr.Data}
 		}
 	}
 
@@ -949,9 +999,9 @@ func CreateXMLConsumers(cmds *Block, parent, hd, tl string, transform map[string
 		return nil
 	}
 
-	out := make(chan XMLRecord, ChanDepth())
+	out := make(chan XMLRecord, chanDepth)
 	if out == nil {
-		fmt.Fprintf(os.Stderr, "\nERROR: Unable to create consumer channel\n")
+		DisplayError("Unable to create consumer channel")
 		os.Exit(1)
 	}
 
@@ -995,7 +1045,7 @@ func CreateXMLConsumers(cmds *Block, parent, hd, tl string, transform map[string
 	var wg sync.WaitGroup
 
 	// launch multiple consumer goroutines
-	for i := 0; i < NumServe(); i++ {
+	for range numServe {
 		wg.Add(1)
 		go xmlConsumer(cmds, parent, &wg, inp, out)
 	}
@@ -1020,9 +1070,9 @@ func CreateSelectors(parent, indx string, order map[string]bool, inp <-chan XMLR
 
 	find := ParseIndex(indx)
 
-	out := make(chan XMLRecord, ChanDepth())
+	out := make(chan XMLRecord, chanDepth)
 	if out == nil {
-		fmt.Fprintf(os.Stderr, "\nERROR: Unable to create selector channel\n")
+		DisplayError("Unable to create selector channel")
 		os.Exit(1)
 	}
 
@@ -1062,7 +1112,7 @@ func CreateSelectors(parent, indx string, order map[string]bool, inp <-chan XMLR
 	var wg sync.WaitGroup
 
 	// launch multiple selector goroutines
-	for i := 0; i < NumServe(); i++ {
+	for range numServe {
 		wg.Add(1)
 		go xmlSelector(&wg, inp, out)
 	}
@@ -1083,9 +1133,9 @@ func CreateUnicoders(inp <-chan XMLRecord) <-chan XMLRecord {
 		return nil
 	}
 
-	out := make(chan XMLRecord, ChanDepth())
+	out := make(chan XMLRecord, chanDepth)
 	if out == nil {
-		fmt.Fprintf(os.Stderr, "\nERROR: Unable to create selector channel\n")
+		DisplayError("Unable to create selector channel")
 		os.Exit(1)
 	}
 
@@ -1114,7 +1164,7 @@ func CreateUnicoders(inp <-chan XMLRecord) <-chan XMLRecord {
 	var wg sync.WaitGroup
 
 	// launch multiple unicoder goroutines
-	for i := 0; i < NumServe(); i++ {
+	for range numServe {
 		wg.Add(1)
 		go xmlUnicoder(&wg, inp, out)
 	}
@@ -1313,38 +1363,16 @@ func DrainExtractions(head, tail, posn string, mpty, idnt bool, histogram map[st
 	wrtr.Flush()
 
 	// print -histogram results, if populated
-	var keys []string
-	for ky := range histogram {
-		keys = append(keys, ky)
-	}
-	if len(keys) > 0 {
-		// sort fields in alphabetical or numeric order
-		sort.Slice(keys, func(i, j int) bool {
-			// numeric sort on strings checks lengths first
-			if IsAllDigits(keys[i]) && IsAllDigits(keys[j]) {
-				lni := len(keys[i])
-				lnj := len(keys[j])
-				// shorter string is numerically less, assuming no leading zeros
-				if lni < lnj {
-					return true
-				}
-				if lni > lnj {
-					return false
-				}
-			}
-			// same length or non-numeric, can now do string comparison on contents
-			return keys[i] < keys[j]
-		})
+	keys := slices.SortedFunc(maps.Keys(histogram), CompareAlphaOrNumericKeys)
 
-		for _, str := range keys {
+	for _, str := range keys {
 
-			count := histogram[str]
-			val := strconv.Itoa(count)
-			os.Stdout.WriteString(val)
-			os.Stdout.WriteString("\t")
-			os.Stdout.WriteString(str)
-			os.Stdout.WriteString("\n")
-		}
+		count := histogram[str]
+		val := strconv.Itoa(count)
+		os.Stdout.WriteString(val)
+		os.Stdout.WriteString("\t")
+		os.Stdout.WriteString(str)
+		os.Stdout.WriteString("\n")
 	}
 
 	// force garbage collection and return memory before calculating processing rate
@@ -1394,13 +1422,13 @@ const (
 
 // XMLNode is the node for an internal tree structure representing a single XML record
 type XMLNode struct {
+	Attribs    []string
+	Children   *XMLNode
+	Next       *XMLNode
 	Name       string
 	Parent     string
 	Contents   string
 	Attributes string
-	Attribs    []string
-	Children   *XMLNode
-	Next       *XMLNode
 }
 
 // XMLFind contains individual field values for finding a particular object
@@ -1414,10 +1442,10 @@ type XMLFind struct {
 
 // XMLToken is the unit of XML parsing
 type XMLToken struct {
-	Tag   int
-	Cont  int
 	Name  string
 	Attr  string
+	Tag   int
+	Cont  int
 	Index int
 	Line  int
 }
@@ -1435,7 +1463,7 @@ func ParseAttributes(attrb string) []string {
 	num := 0
 	inQuote := false
 
-	for i := 0; i < attlen; i++ {
+	for i := range attlen {
 		ch := attrb[i]
 		if ch == '"' || ch == '\'' {
 			// "
@@ -1493,7 +1521,7 @@ func ParseAttributes(attrb string) []string {
 				}
 				if ch != '"' && ch != '\'' {
 					// "
-					fmt.Fprintf(os.Stderr, "\nAttribute in '%s' missing double quote\n", attrb)
+					DisplayError("Attribute in '%s' missing double quote", attrb)
 				}
 			}
 			// skip past leading double quote
@@ -1708,7 +1736,7 @@ func parseXML(record, parent string, inp <-chan XMLBlock, tokens func(XMLToken),
 							ch = text[idx]
 						}
 						if ch != '>' {
-							fmt.Fprintf(os.Stderr, "\nSelf-closing element missing right angle bracket\n")
+							DisplayError("Self-closing element missing right angle bracket")
 						}
 					}
 					idx++
@@ -1731,7 +1759,7 @@ func parseXML(record, parent string, inp <-chan XMLBlock, tokens func(XMLToken),
 						ch = text[idx]
 					}
 					if ch != '>' {
-						fmt.Fprintf(os.Stderr, "\nAttributes not followed by right angle bracket\n")
+						DisplayError("Attributes not followed by right angle bracket")
 					}
 					// walk back past trailing blanks
 					lst := idx - 1
@@ -1755,9 +1783,9 @@ func parseXML(record, parent string, inp <-chan XMLBlock, tokens func(XMLToken),
 				} else {
 
 					if countLines {
-						fmt.Fprintf(os.Stderr, "\nUnexpected punctuation '%c' in XML element, line %d\n", ch, currentLineCount(idx))
+						DisplayError("Unexpected punctuation '%c' in XML element, line %d", ch, currentLineCount(idx))
 					} else {
-						fmt.Fprintf(os.Stderr, "\nUnexpected punctuation '%c' in XML element\n", ch)
+						DisplayError("Unexpected punctuation '%c' in XML element", ch)
 					}
 
 					return STARTTAG, NONE, str[:], "", idx
@@ -1787,7 +1815,7 @@ func parseXML(record, parent string, inp <-chan XMLBlock, tokens func(XMLToken),
 							ch = text[idx]
 						}
 						if ch != '>' {
-							fmt.Fprintf(os.Stderr, "\nUnexpected characters after end element name\n")
+							DisplayError("Unexpected characters after end element name")
 						}
 					}
 					idx++
@@ -1796,9 +1824,9 @@ func parseXML(record, parent string, inp <-chan XMLBlock, tokens func(XMLToken),
 				}
 				// legal character not found after slash
 				if countLines {
-					fmt.Fprintf(os.Stderr, "\nUnexpected punctuation '%c' in XML element, line %d\n", ch, currentLineCount(idx))
+					DisplayError("Unexpected punctuation '%c' in XML element, line %d", ch, currentLineCount(idx))
 				} else {
-					fmt.Fprintf(os.Stderr, "\nUnexpected punctuation '%c' in XML element\n", ch)
+					DisplayError("Unexpected punctuation '%c' in XML element", ch)
 				}
 
 			} else if ch == '!' {
@@ -1821,10 +1849,11 @@ func parseXML(record, parent string, inp <-chan XMLBlock, tokens func(XMLToken),
 					which = DOCTYPETAG
 					skipTo = ">"
 				}
+				idx = start
 				if which != NOTAG && skipTo != "" {
 					whch := which
 					// CDATA or COMMENT block may contain internal angle brackets
-					found := strings.Index(text[idx:], skipTo)
+					found := strings.Index(text[start:], skipTo)
 					if found < 0 {
 						// string stops in middle of CDATA or COMMENT
 						if inp != nil {
@@ -1936,9 +1965,9 @@ func parseXML(record, parent string, inp <-chan XMLBlock, tokens func(XMLToken),
 			} else {
 
 				if countLines {
-					fmt.Fprintf(os.Stderr, "\nUnexpected punctuation '%c' (%d) in XML element, line %d\n", ch, ch, currentLineCount(idx))
+					DisplayError("Unexpected punctuation '%c' (%d) in XML element, line %d", ch, ch, currentLineCount(idx))
 				} else {
-					fmt.Fprintf(os.Stderr, "\nUnexpected punctuation '%c' (%d) in XML element\n", ch, ch)
+					DisplayError("Unexpected punctuation '%c' (%d) in XML element", ch, ch)
 				}
 			}
 
@@ -2095,9 +2124,9 @@ func parseXML(record, parent string, inp <-chan XMLBlock, tokens func(XMLToken),
 
 			if tag == BADTAG {
 				if countLines {
-					fmt.Fprintf(os.Stderr, "\n%sERROR: Unparsable XML element, line %d%s\n", RED, lineNum, INIT)
+					DisplayError("Unparsable XML element, line %d", lineNum)
 				} else {
-					fmt.Fprintf(os.Stderr, "\n%sERROR: Unparsable XML element%s\n", RED, INIT)
+					DisplayError("Unparsable XML element")
 				}
 				break
 			}
@@ -2108,7 +2137,7 @@ func parseXML(record, parent string, inp <-chan XMLBlock, tokens func(XMLToken),
 			switch tag {
 			case STARTTAG:
 				if status == CHAR {
-					fmt.Fprintf(os.Stderr, "%s ERROR: %s UNEXPECTED MIXED CONTENT <%s> IN <%s>%s\n", INVT, LOUD, name, prnt, INIT)
+					DisplayError("UNEXPECTED MIXED CONTENT <%s> IN <%s>", name, prnt)
 				}
 				// read sub tree
 				obj, ok = parseSpecial(name, attr, node.Name)
@@ -2190,9 +2219,9 @@ func parseXML(record, parent string, inp <-chan XMLBlock, tokens func(XMLToken),
 
 			if tag == BADTAG {
 				if countLines {
-					fmt.Fprintf(os.Stderr, "\n%sERROR: Unparsable XML element, line %d%s\n", RED, lineNum, INIT)
+					DisplayError("Unparsable XML element, line %d", lineNum)
 				} else {
-					fmt.Fprintf(os.Stderr, "\n%sERROR: Unparsable XML element%s\n", RED, INIT)
+					DisplayError("Unparsable XML element")
 				}
 				break
 			}
@@ -2204,9 +2233,9 @@ func parseXML(record, parent string, inp <-chan XMLBlock, tokens func(XMLToken),
 			case STARTTAG:
 				if status == CHAR {
 					if doStrict {
-						fmt.Fprintf(os.Stderr, "%s ERROR: %s UNRECOGNIZED MIXED CONTENT <%s> IN <%s>%s\n", INVT, LOUD, name, prnt, INIT)
+						DisplayError("UNRECOGNIZED MIXED CONTENT <%s> IN <%s>", name, prnt)
 					} else if !doMixed {
-						fmt.Fprintf(os.Stderr, "%s ERROR: %s UNEXPECTED MIXED CONTENT <%s> IN <%s>%s\n", INVT, LOUD, name, prnt, INIT)
+						DisplayError("UNEXPECTED MIXED CONTENT <%s> IN <%s>", name, prnt)
 					}
 				}
 				// read sub tree
@@ -2326,9 +2355,9 @@ func parseXML(record, parent string, inp <-chan XMLBlock, tokens func(XMLToken),
 
 			if tag == BADTAG {
 				if countLines {
-					fmt.Fprintf(os.Stderr, "\n%sERROR: Unparsable XML element, line %d%s\n", RED, lineNum, INIT)
+					DisplayError("Unparsable XML element, line %d", lineNum)
 				} else {
-					fmt.Fprintf(os.Stderr, "\n%sERROR: Unparsable XML element%s\n", RED, INIT)
+					DisplayError("Unparsable XML element")
 				}
 				break
 			}
@@ -2384,14 +2413,14 @@ func parseXML(record, parent string, inp <-chan XMLBlock, tokens func(XMLToken),
 
 			if tag == BADTAG {
 				if countLines {
-					fmt.Fprintf(os.Stderr, "\n%sERROR: Unparsable XML element, line %d%s\n", RED, lineNum, INIT)
+					DisplayError("Unparsable XML element, line %d", lineNum)
 				} else {
-					fmt.Fprintf(os.Stderr, "\n%sERROR: Unparsable XML element%s\n", RED, INIT)
+					DisplayError("Unparsable XML element")
 				}
 				break
 			}
 
-			tkn := XMLToken{tag, ctype, name, attr, idx, lineNum}
+			tkn := XMLToken{Tag: tag, Cont: ctype, Name: name, Attr: attr, Index: idx, Line: lineNum}
 
 			tokens(tkn)
 
@@ -2416,9 +2445,9 @@ func parseXML(record, parent string, inp <-chan XMLBlock, tokens func(XMLToken),
 
 			if tag == BADTAG {
 				if countLines {
-					fmt.Fprintf(os.Stderr, "\n%sERROR: Unparsable XML element, line %d%s\n", RED, lineNum, INIT)
+					DisplayError("Unparsable XML element, line %d", lineNum)
 				} else {
-					fmt.Fprintf(os.Stderr, "\n%sERROR: Unparsable XML element%s\n", RED, INIT)
+					DisplayError("Unparsable XML element")
 				}
 				break
 			}
@@ -2445,9 +2474,9 @@ func parseXML(record, parent string, inp <-chan XMLBlock, tokens func(XMLToken),
 
 		if tag == BADTAG {
 			if countLines {
-				fmt.Fprintf(os.Stderr, "\n%sERROR: Unparsable XML element, line %d%s\n", RED, lineNum, INIT)
+				DisplayError("Unparsable XML element, line %d", lineNum)
 			} else {
-				fmt.Fprintf(os.Stderr, "\n%sERROR: Unparsable XML element%s\n", RED, INIT)
+				DisplayError("Unparsable XML element")
 			}
 			break
 		}
@@ -2544,7 +2573,7 @@ func CreateTokenizer(inp <-chan XMLBlock) <-chan XMLToken {
 
 	out := make(chan XMLToken, chanDepth)
 	if out == nil {
-		fmt.Fprintf(os.Stderr, "\n%sERROR: Unable to create XML tokenizer channel%s\n", RED, INIT)
+		DisplayError("Unable to create XML tokenizer channel")
 		os.Exit(1)
 	}
 
@@ -2719,6 +2748,11 @@ func ExploreNodes(curr *XMLNode, prnt, match string, index, level int, proc func
 		wildcard = true
 	}
 
+	// Single * allows exploration of heterogeneous data construct without knowing current component name
+	if prnt == "" && match == "*" {
+		match = curr.Name
+	}
+
 	// **/Object performs deep exploration of recursive data
 	deep := false
 	if prnt == "**" {
@@ -2789,7 +2823,8 @@ func ExploreNodes(curr *XMLNode, prnt, match string, index, level int, proc func
 // EXPLORE SHORTCUTS
 
 // VisitElements is a shortcut to ExploreElements
-func VisitElements(curr *XMLNode, match string, proc func(string)) {
+// Use mask argument to block exploration into recursive object
+func VisitElements(curr *XMLNode, match, mask string, proc func(string)) {
 
 	if curr == nil || proc == nil {
 		return
@@ -2799,7 +2834,7 @@ func VisitElements(curr *XMLNode, match string, proc func(string)) {
 	prnt, match := SplitInTwoRight(match, "/")
 	match, attrib := SplitInTwoLeft(match, "@")
 
-	ExploreElements(curr, "", prnt, match, attrib, false, false, 0, func(str string, lvl int) { proc(str) })
+	ExploreElements(curr, mask, prnt, match, attrib, false, false, 0, func(str string, lvl int) { proc(str) })
 }
 
 // VisitNodes is a shortcut to ExploreNodes
@@ -2815,3 +2850,44 @@ func VisitNodes(curr *XMLNode, match string, proc func(*XMLNode)) {
 
 	ExploreNodes(curr, prnt, match, 0, 0, func(node *XMLNode, idx, lvl int) { proc(node) })
 }
+
+// STRING TO XML UTILITY
+
+// StringToXML supports XXXtoXML convenience functions
+func StringToXML(rec string, proc func(io.Reader) <-chan string) string {
+
+	if rec == "" || proc == nil {
+		return ""
+	}
+
+	rdr := strings.NewReader(rec)
+	if rdr == nil {
+		DisplayError("Unable to create NewReader")
+		return ""
+	}
+
+	chn := proc(rdr)
+	if chn == nil {
+		DisplayError("Unable to create StringToXML converter")
+		return ""
+	}
+
+	var arry []string
+
+	// drain output of channel
+	for str := range chn {
+		if str == "" {
+			continue
+		}
+
+		if !strings.HasSuffix(str, "\n") {
+			str += "\n"
+		}
+
+		arry = append(arry, str)
+	}
+
+	res := strings.Join(arry, "")
+
+	return res
+}
