@@ -1998,12 +1998,12 @@ func processClause(
 				str = CleanupBadSpaces(str)
 			}
 			if HasAngleOrAmpersandEncoding(str) {
-				str = RepairEncodedMarkup(str)
+				str = CleanupEncodedMarkup(str)
 				str = RepairTableMarkup(str, SPACE)
 				str = RepairScriptMarkup(str, SPACE)
 				str = RepairMathMLMarkup(str, SPACE)
 				// RemoveEmbeddedMarkup must be called before UnescapeString, which was suppressed in ExploreElements
-				str = RemoveEmbeddedMarkup(str)
+				str = RemoveEmbeddedMarkup(str, SPACE)
 			}
 
 			if HasAmpOrNotASCII(str) {
